/*
 * @WriteBrineXMLFile.java Version 1.1 05/22/2014
 *
 * Copyright (c) 2014 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.io;

import java.io.*;
import java.awt.*;

import util.utilFileIO;
import brine.brineListStruct;
import brine.brineStruct;
import cmn.cmnString;

/** Class WriteBrineXMLFile
 *  <p> This class will parse the Brine List Structure and
 *      write the information to the Brine XML File.
 *
 *  @version 1.1 05/22/2014
 *  @author  John R. Victorine
 */

public class WriteBrineXMLFile
{
  private static final String DTD = "<?xml version=\"1.0\"?>\n"       +
  "<!DOCTYPE fluid_data [\n"                     +
  "<!ELEMENT fluid_data (columns*, data*)+>\n"   +
  "<!ELEMENT columns (column*)>\n"  +
  "<!ATTLIST columns records CDATA #REQUIRED>\n" +
  "<!ELEMENT column EMPTY>\n"                    +
  "<!ATTLIST column mnemonic CDATA #REQUIRED\n"  +
  "                 name     CDATA #REQUIRED\n"  +
  "                 unit     CDATA #REQUIRED\n"  +
  "                 type     CDATA #REQUIRED>\n" +
  "<!ELEMENT data (row*)>\n"                     +
  "<!ATTLIST data records CDATA #REQUIRED>\n"    +
  "<!ELEMENT row EMPTY>\n"                   +
  // Primary Keys
  "<!ATTLIST row  KEY     CDATA #IMPLIED\n"  +    // Fluid Primary Key
  "               API     CDATA #IMPLIED\n"  +    // Well API-Number
  "               NAME    CDATA #IMPLIED\n"  +    // Well Name
  "               FIELD   CDATA #IMPLIED\n"  +    // Field
  "               LOC     CDATA #IMPLIED\n"  +    // Location - TRS
  "               COUNTY  CDATA #IMPLIED\n"  +    // County
  "               STATE   CDATA #IMPLIED\n"  +    // State
  // Location Information
  "               LAT     CDATA #IMPLIED\n"  +    // Latitude
  "               LONG    CDATA #IMPLIED\n"  +    // Longitude
   // Elevation
  "               KB      CDATA #IMPLIED\n"  +    // Kelly Bushing
  "               GL      CDATA #IMPLIED\n"  +    // Ground Level
  "               DF      CDATA #IMPLIED\n"  +    // Derrick Floor
  "               TGT     CDATA #IMPLIED\n"  +    // TGT
  // Formation Information
  "               SRC     CDATA #IMPLIED\n"  +    // Sample Source
  "               DATE    CDATA #IMPLIED\n"  +    // Sample Date
  // Sample Information
  "               FORM    CDATA #IMPLIED\n"  +    // Formation
  "               AGE     CDATA #IMPLIED\n"  +    // Formation Age
  // Depth Information
  "               TOP     CDATA #IMPLIED\n"  +    // Depth Top
  "               BASE    CDATA #IMPLIED\n"  +    // Depth Base
  // Other Measured Data
  "               SPGR    CDATA #IMPLIED\n"  +    // Specific Gravity
  "               SIGMA   CDATA #IMPLIED\n"  +    // Specific Conductivity
  "               PH      CDATA #IMPLIED\n"  +    // PH
  "               DEG     CDATA #IMPLIED\n"  +    // Temperature (F)
  "               OHM     CDATA #IMPLIED\n"  +    // Resistivity
  "               OHM75   CDATA #IMPLIED\n"  +    // Resistivity at 75 deg
  "               OHME    CDATA #IMPLIED\n"  +    // Resistivity Estimated
  // Chemical Composition
    // Common Cations

    // .. Alkali Metal
  "               Li      CDATA #IMPLIED\n"  +    // Lithium
  "               Na      CDATA #IMPLIED\n"  +    // Sodium
  "               K       CDATA #IMPLIED\n"  +    // Potassium
  "               Rb      CDATA #IMPLIED\n"  +    // Rubidium
  "               Cs      CDATA #IMPLIED\n"  +    // Cesium
  "               Na_K    CDATA #IMPLIED\n"  +    // Sodium Potassium
    // ..  Alkali Earth Metal
  "               Be      CDATA #IMPLIED\n"  +    // Beryllium
  "               Mg      CDATA #IMPLIED\n"  +    // Magnesium
  "               Ca      CDATA #IMPLIED\n"  +    // Calcium
  "               Sr      CDATA #IMPLIED\n"  +    // Strontium
  "               Ba      CDATA #IMPLIED\n"  +    // Barium
    // ..  VIB
  "               CrII    CDATA #IMPLIED\n"  +    // Chromium (II) chromous
  "               CrIII   CDATA #IMPLIED\n"  +    // Chromium (III) chromic
    // ..  VIIB
  "               MnII    CDATA #IMPLIED\n"  +    // Manganese (II) manganous
  "               MnIII   CDATA #IMPLIED\n"  +    // Manganese (III) manganic
    // ..  VIIIB
  "               FeII    CDATA #IMPLIED\n"  +    // Iron (II) ferrous
  "               FeIII   CDATA #IMPLIED\n"  +    // Iron (III) ferric
  "               CoII    CDATA #IMPLIED\n"  +    // Cobalt (II) cobaltous
  "               CoIII   CDATA #IMPLIED\n"  +    // Cobalt (III) cobaltic
  "               NiII    CDATA #IMPLIED\n"  +    // Nickel (II) nickelous
  "               NiIII   CDATA #IMPLIED\n"  +    // Nickel (III) nickelic
    // ..  IB
  "               Cu      CDATA #IMPLIED\n"  +    // Copper (I) cuprous
  "               CuII    CDATA #IMPLIED\n"  +    // Copper (II) cupric
  "               Ag      CDATA #IMPLIED\n"  +    // Silver
  "               Au      CDATA #IMPLIED\n"  +    // Gold aurous
  "               AuIII   CDATA #IMPLIED\n"  +    // Gold (III) auric
    // ..  IIB
  "               Zn      CDATA #IMPLIED\n"  +    // Zinc
  "               Cd      CDATA #IMPLIED\n"  +    // Cadmium
  "               Hg      CDATA #IMPLIED\n"  +    // Mercury (I) mercurous
  "               HgII    CDATA #IMPLIED\n"  +    // Mercury (II) mercuric
    // ..  IIIA
  "               Al      CDATA #IMPLIED\n"  +    // Aluminum
    //.. VA
  "               SbIII   CDATA #IMPLIED\n"  +    // Antimony (III) antimonous
  "               SbV     CDATA #IMPLIED\n"  +    // Antimony (V) antimonic
  "               BiIII   CDATA #IMPLIED\n"  +    // Bismuth (III) bismuthous,
  "               BiV     CDATA #IMPLIED\n"  +    // Bismuth (V) bismuthic
    // ..  IVA
  "               SnII    CDATA #IMPLIED\n"  +    // Tin (II) stannous
  "               SnIV    CDATA #IMPLIED\n"  +    // Tin (IV) stannic
  "               PbII    CDATA #IMPLIED\n"  +    // Lead (II) plumbous
  "               PbIV    CDATA #IMPLIED\n"  +    // Lead (IV) plumbic
    // .. Other
  "               NH4     CDATA #IMPLIED\n"  +    // Ammonium

    // Common Anions

    // ..Halogen
  "               F       CDATA #IMPLIED\n"  +    // Floride
  "               Cl      CDATA #IMPLIED\n"  +    // Chloride
  "               Br      CDATA #IMPLIED\n"  +    // Bromide
  "               I       CDATA #IMPLIED\n"  +    // Iodine

  "               OH      CDATA #IMPLIED\n"  +    // Hydroxide

  "               BO3     CDATA #IMPLIED\n"  +    // Borate
  "               CO3     CDATA #IMPLIED\n"  +    // Carbonate
  "               HCO3    CDATA #IMPLIED\n"  +    // Bicarbonate

  "               ClO     CDATA #IMPLIED\n"  +    // Hypochlorite
  "               ClO2    CDATA #IMPLIED\n"  +    // Chlorite
  "               ClO3    CDATA #IMPLIED\n"  +    // Chlorate
  "               ClO4    CDATA #IMPLIED\n"  +    // Per chlorate

  "               CN      CDATA #IMPLIED\n"  +    // Cyanide
  "               NCO     CDATA #IMPLIED\n"  +    // Cyanate
  "               OCN     CDATA #IMPLIED\n"  +    // Isocyanate
  "               SCN     CDATA #IMPLIED\n"  +    // Thicyanite

  "               N       CDATA #IMPLIED\n"  +    // Nitride
  "               N3      CDATA #IMPLIED\n"  +    // Azide
  "               NO2     CDATA #IMPLIED\n"  +    // Nitrite
  "               NO3     CDATA #IMPLIED\n"  +    // Nitrate

  "               CrO4    CDATA #IMPLIED\n"  +    // Chromate
  "               Cr2O7   CDATA #IMPLIED\n"  +    // Dichromate

  "               MnO4    CDATA #IMPLIED\n"  +    // Permanganate

  "               P       CDATA #IMPLIED\n"  +    // Phosphide
  "               PO4     CDATA #IMPLIED\n"  +    // Phosphate
  "               HPO4    CDATA #IMPLIED\n"  +    // Monohydrogen Phosphate
  "               H2PO4   CDATA #IMPLIED\n"  +    // Dihydrogen Phosphate

  "               As      CDATA #IMPLIED\n"  +    // Arsinide
  "               Se      CDATA #IMPLIED\n"  +    // Selenide

  "               S       CDATA #IMPLIED\n"  +    // Sulfide
  "               HS      CDATA #IMPLIED\n"  +    // Hydrogen Sulfide
  "               SO3     CDATA #IMPLIED\n"  +    // Sulphite
  "               HSO3    CDATA #IMPLIED\n"  +    // Hydrogen Sulphite
  "               S2O3    CDATA #IMPLIED\n"  +    // Thiosulphate
  "               SO4     CDATA #IMPLIED\n"  +    // Sulfate
  "               HSO4    CDATA #IMPLIED\n"  +    // Bisulfate

    // -- Totals
  // Computed Data
  "               SOLID   CDATA #IMPLIED\n"  +    // Total Solids
  "               TDS     CDATA #IMPLIED>]>\n" + // Total Solids Computed

  "<fluid_data>\n"  +
    "<columns records=\"103\">\n"  +
    "  <column mnemonic=\"KEY\"    name=\"Fluid Primary Key\"      unit=\"NONE\" type=\"KEY\" />\n" +

    // -- Well ID
    "  <column mnemonic=\"API\"    name=\"API-Number\"             unit=\"NONE\" type=\"LOC\" />\n"  +
    "  <column mnemonic=\"NAME\"   name=\"Well Name\"              unit=\"NONE\" type=\"LOC\" />\n"  +

    // -- Other Info
    "  <column mnemonic=\"FIELD\"  name=\"Field\"                  unit=\"NONE\" type=\"LOC\" />\n"  +

    // -- Location
    "  <column mnemonic=\"LOC\"    name=\"Location\"               unit=\"NONE\" type=\"LOC\" />\n"  +
    "  <column mnemonic=\"COUNTY\" name=\"County\"                 unit=\"NONE\" type=\"LOC\" />\n"  +
    "  <column mnemonic=\"STATE\"  name=\"State\"                  unit=\"NONE\" type=\"LOC\" />\n"  +

    "  <column mnemonic=\"LAT\"    name=\"Latitude\"               unit=\"deg\"  type=\"LOC\" />\n"  +
    "  <column mnemonic=\"LONG\"   name=\"Longitude\"              unit=\"deg\"  type=\"LOC\" />\n"  +

    // -- Depth Information
    "  <column mnemonic=\"KB\"     name=\"Kelly Bushing\"          unit=\"ft\"   type=\"ELEV\" />\n" +
    "  <column mnemonic=\"GL\"     name=\"Ground Level\"           unit=\"ft\"   type=\"ELEV\" />\n" +
    "  <column mnemonic=\"DF\"     name=\"Derrick Floor\"          unit=\"ft\"   type=\"ELEV\" />\n" +
    "  <column mnemonic=\"TGT\"    name=\"TGT\"                    unit=\"ft\"   type=\"ELEV\" />\n" +
    // -- Source of Data
    "  <column mnemonic=\"SRC\"    name=\"source\"                 unit=\"NONE\" type=\"SAMPLE\" />\n" +

    "  <column mnemonic=\"FORM\"   name=\"Formation\"              unit=\"NONE\" type=\"FORMATION\" />\n" +
    "  <column mnemonic=\"AGE\"    name=\"Formation Age\"          unit=\"NONE\" type=\"AGE\" />\n" +

    // Depth Range of Sample
    "  <column mnemonic=\"TOP\"    name=\"Depth Top\"              unit=\"ft\"    type=\"DEPTH\" />\n" +
    "  <column mnemonic=\"BASE\"   name=\"Depth Base\"             unit=\"ft\"    type=\"DEPTH\" />\n" +

    // Other Measured Well Data
    "  <column mnemonic=\"SPGR\"   name=\"Specific Gravity\"       unit=\"NONE\" type=\"DATA\" />\n" +
    "  <column mnemonic=\"SIGMA\"  name=\"Specific Conductivity\"  unit=\"mho\"  type=\"COND\" />\n" +
    "  <column mnemonic=\"PH\"     name=\"PH\"                     unit=\"NONE\" type=\"PH\" />\n" +
    "  <column mnemonic=\"DEG\"    name=\"Temperature (F)\"        unit=\"degrees F\" type=\"TEMP\" />\n" +
    "  <column mnemonic=\"OHM\"    name=\"Resistivity (Rw)\"       unit=\"ohm\"  type=\"OHM\"  />\n" +
    "  <column mnemonic=\"OHM75\"  name=\"Resistivity at 75 deg\"  unit=\"ohm\"  type=\"OHM\"  />\n" +
    "  <column mnemonic=\"OHME\"   name=\"Estimated Rw\"           unit=\"ohm\"  type=\"OHM\"  />\n" +

    // Common Cations

    // .. Alkali Metal
    "  <column mnemonic=\"Li\"     name=\"Lithium\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Na\"     name=\"Sodium\"                 unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"K\"      name=\"Potassium\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Rb\"     name=\"Rubidium\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Cs\"     name=\"Cesium\"                 unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Na_K\"   name=\"Sodium Potassium\"       unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  Alkali Earth Metal
    "  <column mnemonic=\"Be\"     name=\"Beryllium\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Mg\"     name=\"Magnesium\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Ca\"     name=\"Calcium\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Sr\"     name=\"Strontium\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Ba\"     name=\"Barium\"                 unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  VIB
    "  <column mnemonic=\"CrII\"   name=\"Chromium (II) chromous\" unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"CrIII\"  name=\"Chromium (III) chromic\" unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  VIIB
    "  <column mnemonic=\"MnII\"   name=\"Manganese (II) manganous\" unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"MnIII\"  name=\"Manganese (III) manganic\" unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  VIIIB
    "  <column mnemonic=\"FeII\"   name=\"Iron (II) ferrous\"      unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"FeIII\"  name=\"Iron (III) ferric\"      unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"CoII\"   name=\"Cobalt (II) cobaltous\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"CoIII\"  name=\"Cobalt (III) cobaltic\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"NiII\"   name=\"Nickel (II) nickelous\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"NiIII\"  name=\"Nickel (III) nickelic\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  IB
    "  <column mnemonic=\"Cu\"     name=\"Copper (I) cuprous\"     unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"CuII\"   name=\"Copper (II) cupric\"     unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Ag\"     name=\"Silver\"                 unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Au\"     name=\"Gold aurous\"            unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"AuIII\"  name=\"Gold (III) auric\"       unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  IIB
    "  <column mnemonic=\"Zn\"     name=\"Zinc\"                   unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Cd\"     name=\"Cadmium\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Hg\"     name=\"Mercury (I) mercurous\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HgII\"   name=\"Mercury (II) mercuric\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  IIIA
    "  <column mnemonic=\"Al\"     name=\"Aluminum\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    //.. VA
    "  <column mnemonic=\"SbIII\"  name=\"Antimony (III) antimonous\" unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"SbV\"    name=\"Antimony (V) antimonic\"    unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"BiIII\"  name=\"Bismuth (III) bismuthous\"  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"BiV\"    name=\"Bismuth (V) bismuthic\"     unit=\"mg/l\" type=\"COMP\" />\n" +
    // ..  IVA
    "  <column mnemonic=\"SnII\"   name=\"Tin (II) stannous\"      unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"SnIV\"   name=\"Tin (IV) stannic\"       unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"PbII\"   name=\"Lead (II) plumbous\"     unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"PbIV\"   name=\"Lead (IV) plumbic\"      unit=\"mg/l\" type=\"COMP\" />\n" +
    // .. Other
    "  <column mnemonic=\"NH4\"    name=\"Ammonium\"               unit=\"mg/l\" type=\"COMP\" />\n" +

    // Common Anions

    // ..Halogen
    "  <column mnemonic=\"F\"      name=\"Floride\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Cl\"     name=\"Chloride\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Br\"     name=\"Bromide\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"I\"      name=\"Iodine\"                 unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"OH\"     name=\"Hydroxide\"              unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"BO3\"    name=\"Borate\"                 unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"CO3\"    name=\"Carbonate\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HCO3\"   name=\"Bicarbonate\"            unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"ClO\"    name=\"Hypochlorite\"           unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"ClO2\"   name=\"Chlorite\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"ClO3\"   name=\"Chlorate\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"ClO4\"   name=\"Per chlorate\"           unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"CN\"     name=\"Cyanide\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"NCO\"    name=\"Cyanate\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"OCN\"    name=\"Isocyanate\"             unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"SCN\"    name=\"Thicyanite\"             unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"N\"      name=\"Nitride\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"N3\"     name=\"Azide\"                  unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"NO2\"    name=\"Nitrite\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"NO3\"    name=\"Nitrate\"                unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"CrO4\"   name=\"Chromate\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Cr2O7\"  name=\"Dichromate\"             unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"MnO4\"   name=\"Permanganate\"           unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"P\"      name=\"Phosphide\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"PO4\"    name=\"Phosphate\"              unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HPO4\"   name=\"Monohydrogen Phosphate\" unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"H2PO4\"  name=\"Dihydrogen Phosphate\"   unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"As\"     name=\"Arsinide\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"Se\"     name=\"Selenide\"               unit=\"mg/l\" type=\"COMP\" />\n" +

    "  <column mnemonic=\"S\"      name=\"Sulfide\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HS\"     name=\"Hydrogen Sulfide\"       unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"SO3\"    name=\"Sulphite\"               unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HSO3\"   name=\"Hydrogen Sulphite\"      unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"S2O3\"   name=\"Thiosulphate\"           unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"SO4\"    name=\"Sulfate\"                unit=\"mg/l\" type=\"COMP\" />\n" +
    "  <column mnemonic=\"HSO4\"   name=\"Bisulfate\"              unit=\"mg/l\" type=\"COMP\" />\n" +

    // -- Totals
    "  <column mnemonic=\"SOLID\"  name=\"Total Solids\"           unit=\"mg/l\" type=\"DATA\" />\n" +
    "  <column mnemonic=\"TDS\"    name=\"Total Solids Computed\"  unit=\"mg/l\" type=\"DATA\" />\n" +
  "</columns>\n";


  /** Method write()
   *  <p> This is the method will write the XML String to the user's PC
   *  @param sPath     = The Directory Path for the XML File
   *  @param sFilename = The Filename of the XML File.
   *  @param sXML      = The Brine Data XML File Contents String
   */

  public static void write(String sPath, String sFilename, String sXML)
  {
    utilFileIO pIO  = new utilFileIO();

    // Open the File for write

    pIO.Open( util.utilFileIO.DIRECTORY,
              util.utilFileIO.WRITE,
              sPath,
              sFilename );

    // Write the XML Block to the File

    pIO.Write( sXML );

    // Close the File

    pIO.Close();
  }

  /** Method buildXML()
   *  <p> This method will build the XML File
   *  @param  st   = brine list data structure
   *  @return sXML = Brine ASCII XML String
   */

  public static String buildXML(brineListStruct st)
  {
    int i,j;
    String      sXML   = "";
    int         iAdded = 0;
    double      data   = 0.0;
    brineStruct stData = null;

    sXML = new String( DTD );

    if (st != null)
    {
  	  sXML = new String( sXML + "<data records=\"" + st.iCount + "\">\n" );

      for (i=0; i<st.iCount; i++)
      {
		stData = st.stItem[i];

        sXML = new String( sXML + "  <row " );

        if (stData.sAPI.length() > 0)
          sXML = new String( sXML + "API=\""    + stData.sAPI       + "\" " );

        if (stData.sName.length() > 0)
          sXML = new String( sXML + "NAME=\""   +
                   cmn.cmnString.convertCharsToHTMLChars(stData.sName)      + "\" " );

        if (stData.sField.length() > 0)
          sXML = new String( sXML + "FIELD=\""  +
                   cmn.cmnString.convertCharsToHTMLChars(stData.sField)     + "\" " );

        if (stData.sLocation.length() > 8)
          sXML = new String( sXML + "LOC=\""    + stData.sLocation  + "\" " );

        if (stData.sCounty.length() > 0)
          sXML = new String( sXML + "COUNTY=\"" + stData.sCounty    + "\" " );

        if (stData.state.length() > 0)
          sXML = new String( sXML + "STATE=\""  + stData.state      + "\" " );

        if (stData.dLatitude != 0.0)
          sXML = new String( sXML + "LAT=\""    + stData.dLatitude  + "\" " );

        if (stData.dLongitude != 0.0)
          sXML = new String( sXML + "LONG=\""   + stData.dLongitude + "\" " );

        if (stData.dKB > 0.0)
          sXML = new String( sXML + "KB=\""     + stData.dKB        + "\" " );

        if (stData.dGL > 0.0)
          sXML = new String( sXML + "GL=\""     + stData.dGL        + "\" " );

        if (stData.dDF > 0.0)
          sXML = new String( sXML + "DF=\""     + stData.dDF        + "\" " );

        if (stData.dTGT > 0.0)
          sXML = new String( sXML + "TGT=\""    + stData.dTGT       + "\" " );

        if (stData.source.length() > 0)
          sXML = new String( sXML + "SRC=\""    + stData.source     + "\" " );

        if (stData.sRecovery.length() > 0)
          sXML = new String( sXML + "DATE=\""   + stData.sRecovery  + "\" " );

        if (stData.sFORM.length() > 0)
          sXML = new String( sXML + "FORM=\""   + stData.sFORM      + "\" " );

        if (stData.sAGE.length() > 0)
          sXML = new String( sXML + "AGE=\""    + stData.sAGE       + "\" " );

		for(j=brine.brineStandardTools._TOP; j<brine.brineStandardTools.TOTAL; j++)
		{
  		  data = st.getData(j, i);
  		  if (j != brine.brineStandardTools._LONG)
		  {
		    if (data < 0.0)
		      data = 0.0;
		  }

		  if (data > 0.0)
		  {
            sXML = new String( sXML + brine.brineStandardTools.CURVES[j][1] + "=\"" + data + "\" " );
		  }
		}

		sXML = new String(  sXML + "/>\n" );
	  }

      sXML = new String( sXML + "</data>\n");
	}

    sXML = new String( sXML + "</fluid_data>");

    return (sXML);
  }
}

/*
 *  @version 1.1 05/22/2014
 *  @author  John Victorine
 */
