/*
 * @brineMath.java Version 1.1 06/07/2011
 *
 * Copyright (c) 2011 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package brine.math;

import brine.brineListStruct;

/** Class brineMath
 *  <p> This Class will perform the basic math for the brine plots.
 *
 *  @version 1.1 06/07/2011
 *  @author  John R. Victorine
 */

public class brineMath
{
  public static final int _CATIONS = 0;
  public static final int _ANIONS  = 1;

  public static final int CATIONS[] =
  {
    brine.brineStandardTools._CA,
    brine.brineStandardTools._MG,
    brine.brineStandardTools._NA,
    brine.brineStandardTools._K,
    brine.brineStandardTools._FE_III,
    brine.brineStandardTools._FE_II,
    brine.brineStandardTools._MN_III,
    brine.brineStandardTools._MN_II,
  };

  public static final int ANIONS[]  =
  {
    brine.brineStandardTools._CL,
    brine.brineStandardTools._BR,
    brine.brineStandardTools._I,
    brine.brineStandardTools._SO4,
    brine.brineStandardTools._HCO3,
    brine.brineStandardTools._CO3,
    brine.brineStandardTools._PO4,
    brine.brineStandardTools._HPO4,
    brine.brineStandardTools._H2PO4,
  };

  /** Method getRows()
   *  <p> This method will return the number of cations or anions
   *  @param  iType = Cations or Anions Indicator
   *  @return iRows = Number of rows;
   */

  public static int getRows( int iType )
  {
    int iRows = 0;

    switch (iType)
    {
	  case _CATIONS:
	    iRows = CATIONS.length;
	    break;

	  case _ANIONS:
	    iRows = ANIONS.length;
	    break;
	}

    return (iRows);
  }

  /** Method get_mg_per_liter()
   *  <p> This method will get the mg/l for either Cations
   *      or Anions from the brine data list structure.
   *  @param  iType = Cations or Anions Indicator
   *  @param  iRow  = Brine Data Row Number
   *  @param  st    = Brine Data List Structure
   *  @return dMG   = Array of mg/l values.
   */

  public static double[] get_mg_per_liter(
	  int iType, int iRow, brineListStruct st )
  {
    double dMG[]  = null;
    int    i      = 0;
    int    len    = 0;

    len = getRows( iType );

    dMG = new double[len];

	// initialize the meq/liter data array

	for (i=0; i<len; i++)
	{
	  dMG[i] = 0.0;
	}

    if (st != null)
    {
	  for (i=0; i<len; i++)
	  {
        switch (iType)
        {
  	      case _CATIONS:
	        dMG[i] = st.getData( CATIONS[i], iRow );
	        break;

	      case _ANIONS:
	        dMG[i] = st.getData( ANIONS[i], iRow );
	        break;
	    }
  	  }
    }

    return (dMG);
  }

  /** Method mg_to_meq_per_liter()
   *  <p> This method will compute meq/l from mg/l for either Cations
   *      or Anions
   *  @param  iType = Cations or Anions Indicator
   *  @param  dMG   = Array of mg/l values.
   *  @return dMEQ  = Array of meq/l values.
   */

  public static double[] mg_to_meq_per_liter(
	  int iType, double dMG[] )
  {
    double dMEQ[]  = null;
    int    i       = 0;
    int    len     = 0;
    int    itmp    = 0;
    double dme_meq = 0.0;

    len  = getRows( iType );

    dMEQ = new double[len];

	// initialize the meq/liter data array

	for (i=0; i<len; i++)
	{
	  dMEQ[i] = 0.0;
	}

    for (i=0; i<len; i++)
    {
      switch (iType)
      {
  	    case _CATIONS:
	      dme_meq = brine.brineStandardTools.getGMperMole(CATIONS[i]) /
	                brine.brineStandardTools.getValence(CATIONS[i]);
	      break;

        case _ANIONS:
          dme_meq = brine.brineStandardTools.getGMperMole(ANIONS[i]) /
                    brine.brineStandardTools.getValence(ANIONS[i]);
	      break;
	  }

      if (dMG[i] > 0.0)
      {
	    dMEQ[i] = dMG[i] / dme_meq;
	    itmp    = (int) (100.0 *dMEQ[i]);
	    dMEQ[i] = itmp / 100.0;
	  }
    }

    return (dMEQ);
  }

  /** Method percent_meq_per_liter()
   *  <p> This method will compute % meq/l from mg/l for either Cations
   *      or Anions
   *  @param  iType = Cations or Anions Indicator
   *  @return dMEQ  = Array of meq/l values.
   *  @return dPER  = Array of % meq/l values.
   */

  public static double[] percent_meq_per_liter(
	  int iType, double dMEQ[] )
  {
    double dPER[]  = null;
    int    itmp    = 0;
    int    i       = 0;
    int    len     = 0;
    double dTotal  = 0.0;

    len  = getRows( iType );

    dPER = new double[len];

	// initialize the Percent data array

	for (i=0; i<len; i++)
	{
	  dPER[i] = 0.0;
	}

    if (dMEQ != null)
    {
	  // Compute the total meq/liter

	  for (i=0; i<len; i++)
	  {
	    dTotal = dTotal + dMEQ[i];
	  }

      // Compute the Percent meq/liter

	  for (i=0; i<len; i++)
	  {
	    dPER[i] = 100.0 * Math.abs(dMEQ[i] / dTotal);
	    itmp    = (int) (100.0 *dPER[i]);
	    dPER[i] = itmp / 100.0;
	  }
	}

	return (dPER);
  }

  /* ======================================================================= *
   * --------------------------- GENERAL METHODS --------------------------- *
   * ------------------------ NOT SPECIFIC TO ORDER ------------------------ *
   * ======================================================================= */

  /** Method get_type()
   *  <p> This method will get the mg/l for either Cations
   *      or Anions from the brine data list structure.
   *  @param  iType = Cations or Anions Indicator
   *  @param  st    = Brine Data List Structure
   *  @return ion   = Array of Standard Brine ID Numbers
   */

  public static int[] get_type( int iType, brineListStruct st )
  {
	int i=0;
	int ion[]  = null;
	int icnt   = 0;
	int iStart = 0;
	int iEnd   = 0;
	int len    = 0;

    switch (iType)
    {
  	  case _CATIONS:
  	    iStart = brine.brineStandardTools._LI;
  	    iEnd   = brine.brineStandardTools._F;
        break;

      case _ANIONS:
  	    iStart = brine.brineStandardTools._F;
  	    iEnd   = brine.brineStandardTools._SOLID;
        break;
    }

	if (st != null)
	{
	  for (i=iStart; i<iEnd; i++)
	  {
        if (st.checkData(i))
          icnt++;
	  }

	  if (icnt > 0)
	  {
		len  = icnt;
		ion  = new int[icnt];
		icnt = 0;

		for (i=iStart; i<iEnd; i++)
		{
		  if (icnt < len)
		  {
            if (st.checkData(i))
            {
              ion[icnt] = i;
              icnt++;
		    }
		  }
		}
	  }
	}

	return (ion);
  }

  /** Method get_spec()
   *  <p> This method will get the mg/l for either Cations
   *      or Anions from the brine data list structure.
   *  @param  iType = Cations or Anions Indicator
   *  @param  st    = Brine Data List Structure
   *  @return ion   = Array of Standard Brine ID Numbers
   */

  public static int[] get_spec( int iType, brineListStruct st )
  {
	int i=0;
	int ion[]  = null;
	int icnt   = 0;
	int iStart = 0;
	int iEnd   = 0;
	int len    = 0;

    switch (iType)
    {
  	  case _CATIONS:
  	    iStart = brine.brineStandardTools._LI;
  	    iEnd   = brine.brineStandardTools._F;
        break;

      case _ANIONS:
  	    iStart = brine.brineStandardTools._F;
  	    iEnd   = brine.brineStandardTools._SOLID;
        break;
    }

	if (st != null)
	{
	  for (i=iStart; i<iEnd; i++)
	  {
        if ((st.checkData(i)) && (checkData(i)))
          icnt++;
	  }

	  if (icnt > 0)
	  {
	    len  = icnt;
	    ion  = new int[icnt];
	    icnt = 0;

  	    for (i=iStart; i<iEnd; i++)
		{
		  if (icnt < len)
		  {
            if ((st.checkData(i)) && (checkData(i)))
            {
              ion[icnt] = i;
              icnt++;
		    }
		  }
		}
	  }
	}

	return (ion);
  }

  /** Method checkData()
   *  <p> This method will check if it is type of anion or cation that should be included.
   *  @param  id   = brine data id number
   *  @return bVal = true it is; false it is not
   */

  private static boolean checkData(int id)
  {
	boolean bVal = false;

	switch (id)
	{
      // .. Alkali Metal
	  case brine.brineStandardTools._LI:      // Lithium            1+
	  case brine.brineStandardTools._NA:      // Sodium             1+
	  case brine.brineStandardTools._K:       // Potassium          1+
	  case brine.brineStandardTools._RB:      // Rubidium           1+
	  case brine.brineStandardTools._CS:      // Cesium             1+
	  //case brine.brineStandardTools._NA_K:    // Sodium/Potasium    1+

      // ..  Alkali Earth Metal
	  //case brine.brineStandardTools._BE:    // Beryllium          2+
	  case brine.brineStandardTools._MG:      // Magnesium          2+
	  case brine.brineStandardTools._CA:      // Calcium            2+
	  case brine.brineStandardTools._SR:      // Strontium          2+
	  case brine.brineStandardTools._BA:      // Barium             2+

      // ..  VIB
	  //case brine.brineStandardTools._CR_II:   // Chromium (II)   chromous  2+
	  //case brine.brineStandardTools._CR_III:  // Chromium (III)  chromic   3+

      // ..  VIIB
	  case brine.brineStandardTools._MN_II:   // Manganese (II)  manganous 2+
	  case brine.brineStandardTools._MN_III:  // Manganese (III) manganic  3+

      // ..  VIIIB
	  case brine.brineStandardTools._FE_II:   // Iron (II)    ferrous      2+
	  case brine.brineStandardTools._FE_III:  // Iron (III)   ferric		 3+
	  //case brine.brineStandardTools._CO_II:   // Cobalt (II)  cobaltous    2+
	  //case brine.brineStandardTools._CO_III:  // Cobalt (III) cobaltic     3+
	  //case brine.brineStandardTools._NI_II:   // Nickel (II)  nickelous    2+
	  //case brine.brineStandardTools._NI_III:  // Nickel (III) nickelic     3+

      // ..  IB
	  //case brine.brineStandardTools._CU_I:    // Copper (I)   cuprous      1+
	  //case brine.brineStandardTools._CU_II:   // Copper (II)  cupric       2+
	  //case brine.brineStandardTools._AG:      // Silver                    1+
	  //case brine.brineStandardTools._AU:      // Gold (I)     aurous       1+
	  //case brine.brineStandardTools._AU_III:  // Gold (III)   auric        3+

      // ..  IIB
	  case brine.brineStandardTools._ZN:      // Zinc                      2+
	  //case brine.brineStandardTools._CD:      // Cadmium                   2+
	  //case brine.brineStandardTools._HG:      // Mercury (I)  mercurous Hg2 2+
	  //case brine.brineStandardTools._HG_II:   // Mercury (II) mercuric     2+

      // ..  IIIA
	  case brine.brineStandardTools._AL:      // Aluminium                 3+

      //.. VA
	  //case brine.brineStandardTools._SB_III:  // Antimony (III)  antimonous  3+
	  //case brine.brineStandardTools._SB_V:    // Antimony (V)    antimonic   5+
	  //case brine.brineStandardTools._BI_III:  // Bismuthous(III) bismuthous  3+
	  //case brine.brineStandardTools._BI_V:    // Bismuth(V)      bismuthic   5+

      // ..  IVA
	  //case brine.brineStandardTools._SN_II:   // Tin (II)  stannous          2+
	  //case brine.brineStandardTools._SN_IV:   // Tin (IV)  stannic           4+
	  //case brine.brineStandardTools._PB_II:   // Lead (II) plumbous          2+
	  //case brine.brineStandardTools._PB_IV:   // Lead (IV) plumbic           4+

      // .. Other
	  case brine.brineStandardTools._NH4:    // Ammonium                    1+

      // Common Anions

	  case brine.brineStandardTools._F:      // Flouride        1-
	  case brine.brineStandardTools._CL:     // Chloride        1-
	  case brine.brineStandardTools._BR:     // Bromide         1-
	  case brine.brineStandardTools._I:      // Iodide          1-

	  //case brine.brineStandardTools._OH:     // Hydroxide       1-

	  case brine.brineStandardTools._BO3:    // Borate
	  case brine.brineStandardTools._CO3:    // Carbonate       2-
	  case brine.brineStandardTools._HCO3:   // Bicarbonate     1-

	  //case brine.brineStandardTools._CLO:    // Hypochlorite    1-
	  //case brine.brineStandardTools._CLO2:   // Chlorite        1-
	  //case brine.brineStandardTools._CLO3:   // Chlorate        1-
	  //case brine.brineStandardTools._CLO4:   // Per chlorate    1-

	  //case brine.brineStandardTools._CN:     // Cyanide         1-
	  //case brine.brineStandardTools._NCO:    // Cyanate         1-
	  //case brine.brineStandardTools._OCN:    // Isocyanate      1-
	  //case brine.brineStandardTools._SCN:    // Thiocyanite     1-

	  //case brine.brineStandardTools._N:      // Nitride         3-
	  //case brine.brineStandardTools._N3:     // Azide           1-
	  case brine.brineStandardTools._NO2:    // Nitrite         1-
	  case brine.brineStandardTools._NO3:    // Nitrate         1-

	  //case brine.brineStandardTools._CRO4:   // Chromium (II)   Chromate    2-
	  //case brine.brineStandardTools._CR2O7:  // Chromium (III)  Dichormate  2-

	  //case brine.brineStandardTools._MNO4:   // Permanganate    1-

	  //case brine.brineStandardTools._P:      // Phosphide       3-
	  //case brine.brineStandardTools._PO3:    // Phosphite       3-
	  case brine.brineStandardTools._PO4:    // Phosphate       3-
	  case brine.brineStandardTools._HPO4:   // Monohydrogen Phosphate 2-
	  case brine.brineStandardTools._H2PO4:  // Dihydrogen Phosphate   1-

	  //case brine.brineStandardTools._AS:     // Arsinide        3-
	  //case brine.brineStandardTools._SE:     // Selenide        2-

	  //case brine.brineStandardTools._S:      // Sulfide            2-
	  //case brine.brineStandardTools._HS:     // Hydrogen Sulfide   1-
	  //case brine.brineStandardTools._SO3:    // Sulphite           2-
	  //case brine.brineStandardTools._HSO3:   // Hydrogen Sulphite  1-
	  //case brine.brineStandardTools._S2O3:   // Thiosulphate       2-
	  case brine.brineStandardTools._SO4:    // Sulfate            2-
	  case brine.brineStandardTools._HSO4:   // Bisulfate          1-
	    bVal = true;
	    break;
    }

	return (bVal);
  }

  /** Method get_mg_per_liter()
   *  <p> This method will get the mg/l for either Cations
   *      or Anions from the brine data list structure.
   *  @param  iType = Cations or Anions Indicator
   *  @param  iRow  = Brine Data Row Number
   *  @param  ion   = Array of Standard Brine ID Numbers
   *  @param  st    = Brine Data List Structure
   *  @return dMG   = Array of mg/l values.
   */

  public static double[] get_mg_per_liter(
	  int iType, int iRow, int ion[], brineListStruct st )
  {
    double dMG[]  = null;
    int    i      = 0;
    int    len    = 0;

    len = ion.length;
    dMG = new double[len];

	// initialize the meq/liter data array

	for (i=0; i<len; i++)
	{
	  dMG[i] = 0.0;
	}

    if (st != null)
    {
	  for (i=0; i<len; i++)
	  {
        dMG[i] = st.getData( ion[i], iRow );
  	  }
    }

    return (dMG);
  }

  /** Method mg_to_meq_per_liter()
   *  <p> This method will compute meq/l from mg/l for either Cations
   *      or Anions
   *  @param  iType = Cations or Anions Indicator
   *  @param  ion   = Array of Standard Brine ID Numbers
   *  @param  dMG   = Array of mg/l values.
   *  @return dMEQ  = Array of meq/l values.
   */

  public static double[] mg_to_meq_per_liter(
	  int iType, int ion[], double dMG[] )
  {
    double dMEQ[]  = null;
    int    i       = 0;
    int    len     = 0;
    int    itmp    = 0;
    double dme_meq = 0.0;

    len  = dMG.length;
    dMEQ = new double[len];

	// initialize the meq/liter data array

	for (i=0; i<len; i++)
	{
	  dMEQ[i] = 0.0;
	}

    for (i=0; i<len; i++)
    {
	  dme_meq = brine.brineStandardTools.getGMperMole(ion[i]) /
	            brine.brineStandardTools.getValence(ion[i]);

      if (dMG[i] > 0.0)
      {
	    dMEQ[i] = dMG[i] / dme_meq;
	    itmp    = (int) (100.0 *dMEQ[i]);
	    dMEQ[i] = itmp / 100.0;
	  }
    }

    return (dMEQ);
  }

  /** Method percent_meq_per_liter()
   *  <p> This method will compute % meq/l from mg/l for either Cations
   *      or Anions
   *  @param  iType = Cations or Anions Indicator
   *  @param  iRows = total number of Cations or Anions
   *  @param  dMEQ  = Array of meq/l values.
   *  @return dPER  = Array of % meq/l values.
   */

  public static double[] percent_meq_per_liter(
	  int iType, int iRows, double dMEQ[] )
  {
    double dPER[]  = null;
    int    itmp    = 0;
    int    i       = 0;
    int    len     = 0;
    double dTotal  = 0.0;

    len  = iRows;
    dPER = new double[len];

	// initialize the Percent data array

	for (i=0; i<len; i++)
	{
	  dPER[i] = 0.0;
	}

    if (dMEQ != null)
    {
	  // Compute the total meq/liter

	  for (i=0; i<len; i++)
	  {
	    dTotal = dTotal + dMEQ[i];
	  }

      // Compute the Percent meq/liter

	  for (i=0; i<len; i++)
	  {
	    dPER[i] = 100.0 * Math.abs(dMEQ[i] / dTotal);
	    itmp    = (int) (100.0 *dPER[i]);
	    dPER[i] = itmp / 100.0;
	  }
	}

	return (dPER);
  }

  /** Method mg_to_meq_per_liter()
   *  <p> This method will compute meq/l from mg/l for either Cations
   *      or Anions
   *  @param  ion   = Array of Standard Brine ID Numbers
   *  @param  dMG   = Array of mg/l values.
   *  @return dMEQ  = Array of meq/l values.
   */

  public static double meq_to_mg_per_liter(int ion, double dMEQ )
  {
    double dMG     = 0.0;
    double dme_meq = 0.0;

    dme_meq = brine.brineStandardTools.getGMperMole(ion) /
              brine.brineStandardTools.getValence(ion);

    if (dMEQ > 0.0)
    {
	  dMG  = (double) ((int) (100.0 * dMEQ * dme_meq)) / 100.0;
	}

    return (dMG);
  }
}

/*
 *  @version 1.1 06/07/2011
 *  @author  John Victorine
 */
