/*
 * @pcBrineExpandFrame.java Version 1.1 11/06/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import gui.guiSearchDirectoryFrame; // GUI Search Directory Frame

import brine.brineListStruct;       // Brine List Structure

import pc.pcDataStruct;             // Brine List Structure
import pc.pcPlotFilterListStruct;   // Plot Filter List Structure

import pc.gui.pcBrinePanel;         // Brine Panel
import pc.gui.pcCovarianceFrame;    // Covariance Frame
import pc.gui.pcEigenvectorFrame;   // Eigenvector Frame
import pc.gui.pcPlotXYFrame;        // XY Plot Frame
import pc.gui.pcPlotXplotFrame;     // Plot Principal Components Scores

import plot.plotSymbolStruct;       // Symbol Data Structure
import plot.plotXYStruct;           // XY Data Structure

import Jama.*;                      // Import Jama classes

/** Class pcBrineExpandFrame
 *  <p> This Class will create the Stiff Frame.
 *
 *  @version 1.1 11/06/2015
 *  @author  John R. Victorine
 */

public class pcBrineExpandFrame extends JFrame implements ActionListener, Observer
{
  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  // Input Variables

  private double       dLimit   = 0.0;

  // Global Variables

  private Observable              notifier  = null; // Observables Object
  private pcBrinePanel            pnl       = null; // Brine Panel
  private pcCovarianceFrame       pCov      = null; // Covariance Frame
  private pcEigenvectorFrame      pEigen    = null; // eigenvector Frame

  // -- principle component data structures

  private static final int _ORIGINAL   = 0;
  private static final int _GOOD       = 1;
  private static final int _ROGUE      = 2;
  private static final int _ROGUE2     = 3;
  private static final int _NORMALIZED = 4;
  private static final int _TOTAL      = 5;

  private int iSelected   = _ORIGINAL;

  private int          iOriginal   = 0;
  private int          iGood       = 0;
  private int          iRogue      = 0;
  private int          iRogue1     = 0;
  private int          iRogue2     = 0;

  private pcDataStruct stPCD  = null; // original data organized by Good and Bad
  private pcDataStruct stGood = null; // good data set used for normalization
  private pcDataStruct stBad  = null; // "bad" data set to be normalized to good
  private pcDataStruct stBad1 = null; // Below "bad" data set to be normalized to good
  private pcDataStruct stBad2 = null; // Above "bad" data set to be normalized to good
  private pcDataStruct stNorm = null; // Normalized data organized to reflect the original

  private static final String LABELS[] = { "Original Data", "Good Data", "Rogue Data",
                                           "Rogue Data", "Normalized" };

  private brineListStruct stList = null;

  // -- Directory Search for PDF

  // Save Path & Filename

  private static final int _PDF = 0;
  private static final int _XML = 1;
  private static final int _CSV = 2;

  private int    iSave    = -1;

  private guiSearchDirectoryFrame pSearch   = null; // Search for Directory

  private String sPath    = "";
  private String sName    = "";

  // -- Filter Data By

  private pcPlotFilterListStruct stLegend = null; // Legend Filter List Data Structure

  // -- Plot Control Data

  private String sTitle1  = "Kansas Geological Survey";
  private String sTitle2  = "Brine Data";
  private String sTitle3  = "";

  private String sXAxis   = "Sum of Cations (meq/l)";
  private int    iXLog    = 0;
  private double dXMin    = 2600.0;
  private double dXMax    = 4200.0;
  private double dXIncr   = 200.0;

  private String sYAxis   = "Sum of Anions (meq/l)";
  private int    iYLog    = 0;
  private double dYMin    = 2600.0;
  private double dYMax    = 4200.0;
  private double dYIncr   = 200.0;

  // -- Plot Classes

  private plotXYStruct     stPlotXY = null; // XY Plot Data Structure
  private pcPlotXYFrame    plotXY   = null; // XY Plot Frame

  private plotXYStruct     stPlotPc = null;
  private pcPlotXplotFrame plotXYPc = null; //

  // Global Widgets

  private JRadioButton rb[] = null;

  private JButton btnClose  = null;
  private JButton btnCoval  = null;
  private JButton btnEigen  = null;
  private JButton btnPC_2D  = null;
  private JButton btnPlot   = null;

  private JButton btnPDF    = null;
  private JButton btnCSV    = null;
  private JButton btnXML    = null;

  /** Constructor pcBrineExpandFrame()
   *  <p> This is the Constructor for this class.
   *  @param st       = Brine Plot Data Structure
   *  @param dPercent = Percent from Anions/Cations equal to 1
   */

  public pcBrineExpandFrame( pcDataStruct st, double dPercent )
  {
    try
    {
      this.dLimit = (double) ((int) (10.0 * dPercent) / 1000.0);

      setData( st );

      jbInit();

      addWindowListener( new pcBrineExpandFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int     iWidth    = 650;
	JPanel  pnlTop    = new JPanel();
    JPanel  pnlBTN    = new JPanel();

    JButton btnI      = new JButton();
    JButton btnII     = new JButton();
    JButton btnIII    = new JButton();
    JButton btnIV     = new JButton();

    JPanel      pnlRB = new JPanel();
    ButtonGroup group = new ButtonGroup();

    // Create an Observable

    notifier = new pcBrineExpandFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Brine Data showing data greater than 1.5 standard deviations");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Dialog");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    // -- Display Correlation Matrix Frame Button

    btnCoval = new JButton(createImageIcon(cmn.cmnStruct.COVAL));
    btnCoval.setPreferredSize(new Dimension(29, 29));
    btnCoval.setToolTipText("Display Covalence Matrix");
    btnCoval.addActionListener(this);

    // -- Display Eigenvectors & Eigenvalues Matrix Frame Button

    btnEigen = new JButton(createImageIcon(cmn.cmnStruct.EIGEN));
    btnEigen.setPreferredSize(new Dimension(29, 29));
    btnEigen.setToolTipText("Display Eigenvalues & Eigenvector");
    btnEigen.addActionListener(this);

    btnPC_2D = new JButton(createImageIcon(cmn.cmnStruct.PC2D));
    btnPC_2D.setPreferredSize(new Dimension(29, 29));
    btnPC_2D.setToolTipText("Display Principal Components 2D Crossplot");
    btnPC_2D.addActionListener(this);

    // -- 2nd Separator Button

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    // -- Display Anions vs. Cations Plot Frame Button

    btnPlot = new JButton(createImageIcon(cmn.cmnStruct.PLOT));
    btnPlot.setPreferredSize(new Dimension(29, 29));
    btnPlot.setToolTipText("Display Anions vs. Cations Plot");
    btnPlot.addActionListener(this);

    // -- 3rd Separator Button

    btnIII.setPreferredSize(new Dimension(4, 35));
    btnIII.setText("");

    // -- Display PDF Search Directory Frame Button

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Save all Panels Displayed as PNG Images");
    btnPDF.addActionListener(this);

    btnCSV = new JButton(createImageIcon(cmn.cmnStruct.CSV));
    btnCSV.setPreferredSize(new Dimension(29, 29));
    btnCSV.setToolTipText("Save Selected Data as CSV");
    btnCSV.addActionListener(this);

    btnXML = new JButton(createImageIcon(cmn.cmnStruct.XML));
    btnXML.setPreferredSize(new Dimension(29, 29));
    btnXML.setToolTipText("Save Selected Data as XML");
    btnXML.addActionListener(this);

    // -- 4th Separator Button

    btnIV.setPreferredSize(new Dimension(4, 35));
    btnIV.setText("");

    // -- Data Type to Display in Panels & Frames Panel

//    pnlRB.setLayout(new GridLayout());

    rb = new JRadioButton[_TOTAL];

    for (int i=0; i<_TOTAL; i++)
    {
      rb[i] = new JRadioButton();
      if (i==0) { rb[i].setSelected(true); }

      if ((i != _ROGUE) && (i != _ROGUE2))
  	    rb[i].setText( LABELS[i] );
  	  else if ((i == _ROGUE) && ((iRogue1 <= 3) || (iRogue2 <= 3)))
  	    rb[i].setText( LABELS[i] );
  	  else if ((i == _ROGUE) && (iRogue1 > 3))
  	    rb[i].setText( "Data Below 1" );
  	  else if ((i == _ROGUE2) && (iRogue2 > 3))
  	    rb[i].setText( "Data Above 1" );

	  rb[i].addActionListener(this);

      group.add(rb[i]);

      if (i==0)
      {
		pnlRB.add(rb[i], null);
	  }
      else if ((i>0) && (iGood > 3) && (iRogue > 3))
      {
		switch (i)
		{
          case _ORIGINAL:
          case _GOOD:
          case _NORMALIZED:
          case _ROGUE:
            pnlRB.add(rb[i], null);
            break;
          case _ROGUE2:
            if ((iRogue1 > 3) && (iRogue2 > 3))
            pnlRB.add(rb[i], null);
            break;
		}
/*
		if ((i != _ROGUE) && (i != _ROGUE2))
		  pnlRB.add(rb[i], null);
  	    else if ((i == _ROGUE) && ((iRogue1 <= 3) || (iRogue2 <= 3)))
		  pnlRB.add(rb[i], null);
		else if ((i == _ROGUE) && (iRogue1 > 3))
		  pnlRB.add(rb[i], null);
		else if ((i == _ROGUE2) && (iRogue2 > 3))
		  pnlRB.add(rb[i], null);
*/
	  }
	}

    // Center Panel

    pnl = new pcBrinePanel( notifier, stPCD );

    // Attach Widgets

    this.getContentPane().add(pnlTop,  BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                 BorderLayout.WEST);
    pnlBTN.add(btnClose,               null);
    pnlBTN.add(btnI,                   null);
    pnlBTN.add(btnCoval,               null);
    if (iOriginal > 0)
    {
      pnlBTN.add(btnEigen,             null);
      pnlBTN.add(btnPC_2D,             null);
	}
    pnlBTN.add(btnII,                  null);
    pnlBTN.add(btnPlot,                null);
    pnlBTN.add(btnIII,                 null);
    pnlBTN.add(btnCSV,                 null);
    pnlBTN.add(btnXML,                 null);
    pnlBTN.add(btnPDF,                 null);
    pnlBTN.add(btnIV,                  null);
    pnlBTN.add(pnlRB,                  null);

    this.getContentPane().add(pnl,     BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    iWidth = pnl.getWidth();
    if (iWidth == 0)
      iWidth = 650;

    this.setSize(new Dimension(iWidth, d.height-60));

//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);
    this.setLocation( 10, 10 );

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
      System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** METHOD pcBrineExpandFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class pcBrineExpandFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
//    st       = null;
    notifier = null;

    sPath    = null;
    sName    = null;

    if (stPCD != null)
      stPCD.delete();
    stPCD  = null; // original data organized by Good and Bad

    if (stGood != null)
      stGood.delete();
    stGood = null; // good data set used for normalization

    if (stBad != null)
      stBad.delete();
    stBad  = null; // "bad" data set to be normalized to good

    if (stBad1 != null)
      stBad1.delete();
    stBad1  = null; // "bad" data set to be normalized to good

    if (stBad2 != null)
      stBad2.delete();
    stBad2  = null; // "bad" data set to be normalized to good

    if (stNorm != null)
      stNorm.delete();
    stNorm = null; // Normalized data organized to reflect the original

    if (pnl != null)
      pnl.close();
    pnl = null;

    if (pCov != null)
      pCov.close();
    pCov = null;

    if (pEigen != null)
      pEigen.close();
    pEigen = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (stPlotXY != null)
      stPlotXY.delete();
    stPlotXY   = null;

    if (plotXY != null)
      plotXY.close();
    plotXY = null;

    if (stPlotPc != null)
      stPlotPc.delete();
    stPlotPc = null;

    if (plotXYPc != null)
      plotXYPc.close();
    plotXYPc = null;

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    stList   = null;

    btnClose     = null;
    btnCoval     = null;
    btnEigen     = null;
    btnPC_2D     = null;
    btnPlot      = null;

    btnPDF       = null;
    btnCSV       = null;
    btnXML       = null;

    rb = null;

    dispose();
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getData()
   *  <p> This method will retrieve the data structure for processing
   *  @param  iType = the type of data structure to return
   *  @return st = principle component data structure
   */

  private pcDataStruct getData(int iType)
  {
	pcDataStruct st = null;

	switch (iType)
	{
      case _ORIGINAL:
        st = stPCD;
        break;
      case _GOOD:
        st = stGood;
        break;
      case _ROGUE:
        if ((iRogue1 <= 3) || (iRogue2 <= 3))
          st = stBad;
        else
          st = stBad1;
        break;
      case _ROGUE2:
        st = stBad2;
        break;
      case _NORMALIZED:
        st = stNorm;
        break;
	}

	return (st);
  }

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will set the Brine List Data Structure so the normalized
   *      data can be applied to the original data
   *  @param stList = Brine List Structure
   */

  public void setData(brineListStruct stList) {	this.stList = stList; }

  /* ======================================================================= *
   * ----------------------- NORMALIZE DATA METHODS ------------------------ *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will automatically split the data into "Good" and "Rogue"
   *      data sets as well as normalizing the data.  The program is set up
   *      for the user to analyze the data after the fact.
   *  @param st = principle component data structure
   */

  private void setData( pcDataStruct st )
  {
	int i=0;
    Matrix m          = null;
    Matrix evectors   = null;
    EigenvalueDecomposition eigen = null;
    double dEigen[]   = null;
    double dLimits[]  = null;
    int    iColumns[] = null;

    pcDataStruct stG  = null; // good data set used for normalization
    pcDataStruct stB  = null; // "bad" data set to be normalized to good
    pcDataStruct stB1 = null; // "bad" data set to be normalized to good
    pcDataStruct stB2 = null; // "bad" data set to be normalized to good

	if (st != null)
	{
	  dLimits = pc.pcUtility.getLimits( st );
      dXMin   = dLimits[0];
      dXMax   = dLimits[1];
      dXIncr  = dLimits[2];
      dYMin   = dLimits[3];
      dYMax   = dLimits[4];
      dYIncr  = dLimits[5];

      iOriginal = st.iRows;
      iGood     = pc.pcUtility.count_good( dLimit, st );
      iRogue    = pc.pcUtility.count_rogue( dLimit, st );
      iRogue1   = pc.pcUtility.count_rogueBelow( dLimit, st );
      iRogue2   = pc.pcUtility.count_rogueAbove( dLimit, st );

      iColumns  = pc.pcUtility.isData( st );
      iOriginal = checkData( iColumns, iOriginal, st );

      if (iGood > 3)
      {
        iColumns = pc.pcUtility.isData_good( dLimit, st );
        iGood    = checkData( iColumns, iGood, st );
	  }

      if (iRogue > 3)
      {
        iColumns = pc.pcUtility.isData_rogue( dLimit, st );
        iRogue    = checkData( iColumns, iRogue, st );
	  }

      if (iRogue1 > 3)
      {
        iColumns = pc.pcUtility.isData_rogueBelow( dLimit, st );
        iRogue1  = checkData( iColumns, iRogue1, st );
	  }

      if (iRogue2 > 3)
      {
        iColumns = pc.pcUtility.isData_rogueAbove( dLimit, st );
        iRogue2  = checkData( iColumns, iRogue2, st );
	  }

      if ((iGood > 3) && (iRogue > 3))
      {
	    // Get the Good Data Set from the Original Data Set

        stGood   = pc.pcUtility.getBrineData_good( dLimit, st );

        // Get the Rogue Data Set from the Original Data Set

        stBad    = pc.pcUtility.getBrineData_rogue( dLimit, st );

        if (iRogue1 > 3)
          stBad1 = pc.pcUtility.getBrineData_rogueBelow( dLimit, st );

        if (iRogue2 > 3)
          stBad2 = pc.pcUtility.getBrineData_rogueAbove( dLimit, st );

        // Merge the Good and Rogue Data Set into one data set, just reordering
        // the orignal data set with the Good at the top of the list and rogue below it.

        if ((iRogue1 <= 3) || (iRogue2 <= 3))
          stPCD  = pc.pcUtility.mergeBrineData( stGood, stBad );
        else
        {
          stPCD  = pc.pcUtility.mergeBrineData( stGood, stBad1 );
          stPCD  = pc.pcUtility.mergeBrineData( stPCD, stBad2 );
	    }

        // Compute the Eigenvectors & Eigenvalues for the Good Data Set

	    m        = pc.pcUtility.covariance( stGood.iRows, stGood.iColumns, stGood.dAdjust );
	    eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
        dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	    evectors = eigen.getV();				  // Get Eigenvectors

        // Compute the Good Data Set with the Good Eigenvectors & Eigenvalues

        stG      = pc.pcUtility.compute( stGood, stGood.dAdjust, evectors );

        // Compute the Rogue Data Set with the Good Eigenvectors & Eigenvalues & Good Mean

        stB      = pc.pcUtility.compute( stBad, stGood.dataM, stBad.dAdjust, evectors );

        if (iRogue1 > 3)
          stB1   = pc.pcUtility.compute( stBad1, stGood.dataM, stBad1.dAdjust, evectors );
        if (iRogue2 > 3)
          stB2   = pc.pcUtility.compute( stBad2, stGood.dataM, stBad2.dAdjust, evectors );

        // Merge the New Good & New Rogue into one Data set

        if ((iRogue1 <= 3) || (iRogue2 <= 3))
          stNorm = pc.pcUtility.mergeBrineData( stG, stB );
        else
        {
          stNorm = pc.pcUtility.mergeBrineData( stG, stB1 );
          stNorm = pc.pcUtility.mergeBrineData( stNorm, stB2 );
	    }
	  }
	  else
	  {
		stPCD = pc.pcDataUtility.copy( st );
	  }

      if (stG != null)
        stG.delete();
      stG = null;

      if (stB != null)
        stB.delete();
      stB = null;

      if (stB1 != null)
        stB1.delete();
      stB1 = null;

      if (stB2 != null)
        stB2.delete();
      stB2 = null;

      st = null;
	}
  }

  /** Method checkData()
   *  <p> This method will check to see if data columns are absent and display a
   *      warning to the user.
   *  @param iColumns = array of indicators that the column data exists or not
   *  @param iValue   = the number of data within data set that is checked.
   */

  private int checkData(int iColumns[], int iValue, pcDataStruct st )
  {
    String sColumns   = "Columns with no data is[are] ";

    for (int i=0; i<iColumns.length; i++)
    {
	  if (iColumns[i] < 0)
	  {
	    iValue = 0;
	    sColumns = new String(sColumns + " " + CURVES[st.iBrine[i][1]][1] + " ");
	  }
	}

    if (iValue == 0)
      JOptionPane.showMessageDialog((Component) null,
                                    sColumns,
                                    "WARNING",
                                    JOptionPane.WARNING_MESSAGE);
    return (iValue);
  }

  /** Method setData()
   *  <p> This method will set the data to be displayed
   *  @param  iType = the type of data structure to return
   */

  private void setData(int iType)
  {
    Matrix                  m           = null;
    Matrix                  evectors    = null;
    EigenvalueDecomposition eigen       = null;
    double                  data[][]    = null;
    double                  dEigen[]    = null;
    double                  dVector[][] = null;
    String                  sVector[]   = null;
    pcDataStruct            st          = null;

    this.iSelected = iType;

	st = getData( iType );

    if (pnl != null)
    {
	  pnl.setData( st );
	}

    if (pCov != null)
    {
	  m    = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  data = m.getArray();
	  pCov.setData( data );
	}

    if (pEigen != null)
    {
	  m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
      dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	  evectors = eigen.getV();				  // Get Eigenvectors

	  data     = evectors.getArray();
	  pEigen.setData( data, dEigen );
	}

	if (plotXY != null)
	{
      stPlotXY = pc.pcPlotUtility.setPlotStruct( st,
                                                 sTitle1, sTitle2, sTitle3,
                                                 sXAxis, dXMin, dXMax, dXIncr,
                                                 sYAxis, dYMin, dYMax, dYIncr );

      plotXY.setData( stPlotXY );
      plotXY.setTitle3( rb[iSelected].getText() );
	}

    if (plotXYPc != null)
    {
      m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
      eigen    = m.eig();				  // Compute Eigenvalues & Eigenvectors
      evectors = eigen.getV();			  // Get Eigenvectors

      data     = evectors.getArray();

   	  dVector = new double[st.iColumns][2];
	  sVector = new String[st.iColumns];

	  for (int i=0; i<st.iColumns; i++)
	  {
	    dVector[i][1] = data[i][st.iColumns-2];
	    dVector[i][0] = data[i][st.iColumns-1];
	    sVector[i]    = new String(CURVES[st.iBrine[i][1]][1]);
	  }

	  data = computePC_2D( st );

	  stPlotPc = pc.pcPlotUtility.setPlotStruct( st.iRows, 2, data );
	  plotXYPc.setData( stPlotPc );
      plotXYPc.setVectors( st.iColumns, dVector, sVector );
      plotXYPc.setTitle3( rb[iSelected].getText() );
	}
  }

  /* ======================================================================= *
   * --------------------------- DISPLAY FRAMES ---------------------------- *
   * ======================================================================= */

  /** Method buildCovalenceFrame()
   *  <p> This method will build the Covalence Table Frame
   *  @param  iType = the type of data structure to return
   */

  private void buildCovalenceFrame(int iType)
  {
    Matrix m        = null;
    double data[][] = null;
    pcDataStruct st = null;

    if (pCov != null)
      pCov.close();
    pCov = null;

	st = getData( iType );

	if (st != null)
	{
	  m    = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  data = m.getArray();
	  pCov = new pcCovarianceFrame( notifier, st, data );
	}
  }

  /** Method buildEigenvectorFrame()
   *  <p> This method will build the Eigenvector Table Frame
   *  @param  iType = the type of data structure to return
   */

  private void buildEigenvectorFrame(int iType)
  {
    Matrix m        = null;
    Matrix evectors = null;
    EigenvalueDecomposition eigen = null;
    double data[][] = null;
    double dEigen[] = null;
    pcDataStruct st = null;

    if (pEigen != null)
      pEigen.close();
    pEigen = null;

	st = getData( iType );

	if (st != null)
	{
	  m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
      dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	  evectors = eigen.getV();				  // Get Eigenvectors

	  data     = evectors.getArray();
	  pEigen   = new pcEigenvectorFrame( notifier, st, data, dEigen );
	}
  }

  /** Method void display2DPlot()
   *  <p> This method will build the 2D Principal Component Crossplot
   *  @param  iType = the type of data structure to return
   */

  private void display2DPlot(int iType)
  {
//	plotXYStruct stPlotPc = null;
    Matrix m              = null;
    Matrix evectors       = null;
    EigenvalueDecomposition eigen = null;
    double data[][]       = null;
    double dVector[][]    = null;
    String sVector[]      = null;
    pcDataStruct st       = null;

    if (plotXYPc != null)
      plotXYPc.close();
    plotXYPc = null;

	st       = getData( iType );

    m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
    eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
    evectors = eigen.getV();			  // Get Eigenvectors

    data     = evectors.getArray();

	dVector = new double[st.iColumns][2];
	sVector = new String[st.iColumns];

	for (int i=0; i<st.iColumns; i++)
	{
	  dVector[i][1] = data[i][st.iColumns-2];
	  dVector[i][0] = data[i][st.iColumns-1];
	  sVector[i]    = new String(CURVES[st.iBrine[i][1]][1]);
	}

	data = computePC_2D( st );

	stPlotPc = pc.pcPlotUtility.setPlotStruct( st.iRows, 2, data );
    plotXYPc = new pcPlotXplotFrame( notifier, 0, stPlotPc );
    plotXYPc.setVectors(st.iColumns, dVector, sVector);
  }

  /** Method computePC_2D()
   *  <p> This method will build the 2D Principal Component Crossplot
   *  @return dTemp = The first 2 principal components
   */

  private double[][] computePC_2D( pcDataStruct st )
  {
    Matrix m         = null;
    Matrix evectors  = null;
    Matrix evectorsT = null;
    EigenvalueDecomposition eigen = null;
    double data[][]  = null;
    double dTemp[][] = null;
    Matrix pcV       = null;
    Matrix mAdjust   = null;
    Matrix mAdjustT  = null;
    Matrix pcAdjust  = null;
    Matrix pcAdjustT = null;
    double dAdjust[][] = null;
    double dEigen[]  = null;
    int    icnt      = 0;

	if (st != null)
	{
	  m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
      dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	  evectors = eigen.getV();				  // Get Eigenvectors

	  data     = evectors.getArray();

	  dTemp = new double[st.iColumns][2];
	  for (int i=0; i<st.iColumns; i++)
	  {
		icnt = 0;
		for (int j=st.iColumns-2; j<st.iColumns; j++)
		{
          dTemp[i][icnt] = data[i][j];
          icnt++;
		}
	  }

      dAdjust = new double[st.iRows][st.iColumns];
      for (int i=0; i<st.iRows; i++)
      {
		for (int j=0; j<st.iColumns; j++)
		{
		  dAdjust[i][j] = st.dAdjust[i][j] / st.dSD[j];
		}
	  }

      pcV       = pc.pcUtility.setMatrix( st.iColumns, 2, dTemp );
      mAdjust   = pc.pcUtility.setMatrix( st.iRows, st.iColumns, dAdjust );
      mAdjustT  = mAdjust.transpose();

      pcAdjust  = evectors.times(mAdjustT);
      pcAdjustT = pcAdjust.transpose();

      dTemp     = pcAdjustT.getArray();

//	  for (int i=0; i<st.iRows; i++)
//	  {
//  	    System.out.println(i+" "+dTemp[i][0]+" "+dTemp[i][1]);
//	  }
	}

	return (dTemp);
  }

  /* ======================================================================= *
   * ----------------------------- Save Plots ------------------------------ *
   * ======================================================================= */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   *

  private void searchFrame()
  {
	if (sPath.length() == 0)
      sPath = pc.io.pcIO.getHomeDir();

    sName   = new String( "m" );
    pSearch = new guiSearchDirectoryFrame(notifier, sPath, "m", ".png");
  }
*/
  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void createPDF()
  {
	if (pnl != null)
	{
      pc.io.pcIO_Files.createImage( sPath, sName + "_brine", pnl.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName  + "_brine" );
	}

    if (pCov != null)
    {
      pc.io.pcIO_Files.createImage( sPath, sName + "_cov", pCov.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_cov" );
	}

	if (pEigen != null)
	{
      pc.io.pcIO_Files.createImage( sPath, sName + "_eign", pEigen.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_eign" );
	}

	if (plotXY != null)
	{
	  pc.io.pcIO_Files.createImage( sPath, sName + "_plot", plotXY.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_plot" );
	}

    if (plotXYPc != null)
    {
	  pc.io.pcIO_Files.createImage( sPath, sName + "_Xplot", plotXYPc.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_Xplot" );
	}
  }

  /* ===================================================================== *
   * -------------------------- CREATE FILES ----------------------------- *
   * ===================================================================== */

  /** Method createXML()
   *  <p> This method will create a ASCII XML File of Production Data
   */

  private void createXML()
  {
	String sXML = "";
	brineListStruct stNew  = null;
    pcDataStruct    st     = null;

	switch (iSelected)
	{
      case _ORIGINAL:
        stNew = brine.brineUtility.copyList( stList );
        break;
      case _GOOD:
        stNew = pc.pcUtility.applyGroup( stList, stGood );
        break;
      case _ROGUE:
        if ((iRogue1 <= 3) || (iRogue2 <= 3))
          st = stBad;
        else
          st = stBad1;
        stNew = pc.pcUtility.applyGroup( stList, st );
        break;
      case _ROGUE2:
        st = stBad2;
        stNew = pc.pcUtility.applyGroup( stList, stBad2 );
        break;
      case _NORMALIZED:
        stNew = pc.pcUtility.applyNormilization( stList, stNorm );
        break;
	}

	sXML = brine.io.WriteBrineXMLFile.buildXML( stNew );
	brine.io.WriteBrineXMLFile.write( sPath, sName+".xml", sXML );

	if (stNew != null)
	  stNew.delete();
	stNew = null;
  }

  /** Method createCSV()
   *  <p> This method will create a ASCII CSV File of Production Data
   */

  private void createCSV()
  {
	String          sCSV = "";
	brineListStruct stNew  = null;
    pcDataStruct    stData = null;

	switch (iSelected)
	{
      case _ORIGINAL:
        stNew = brine.brineUtility.copyList( stList );
        break;
      case _GOOD:
        stNew = pc.pcUtility.applyGroup( stList, stGood );
        break;
      case _ROGUE:
        if ((iRogue1 <= 3) || (iRogue2 <= 3))
          stData = stBad;
        else
          stData = stBad1;
        stNew = pc.pcUtility.applyGroup( stList, stData );
        break;
      case _ROGUE2:
        stNew = pc.pcUtility.applyGroup( stList, stBad2 );
        break;
      case _NORMALIZED:
        stNew = pc.pcUtility.applyNormilization( stList, stNorm );
        break;
	}

	sCSV = brine.io.WriteBrineCSVFile.buildCSV( stNew );
	brine.io.WriteBrineCSVFile.write( sPath, sName+".csv", sCSV );

	if (stNew != null)
	  stNew.delete();
	stNew = null;
  }

  /* ===================================================================== *
   * ------------------------- PLOT METHODS ------------------------------ *
   * ===================================================================== */

  /** Method display_plot()
   *  <p> This method will create the 3D Plot
   *  @param  iType = the type of data structure to return
   */

  private void display_plot( int iType )
  {
    pcDataStruct st = getData( iType );

	if (plotXY != null)
	  plotXY.close();
	plotXY = null;

    stPlotXY = pc.pcPlotUtility.setPlotStruct( st,
                                               sTitle1, sTitle2, sTitle3,
                                               sXAxis, dXMin, dXMax, dXIncr,
                                               sYAxis, dYMin, dYMax, dYIncr );

    plotXY   = new pcPlotXYFrame( notifier, 0, stPlotXY );
    plotXY.setTitle3( rb[iSelected].getText() );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnCoval)  { buildCovalenceFrame( iSelected ); }
    if (event.getSource() == btnEigen)  { buildEigenvectorFrame( iSelected ); }
    if (event.getSource() == btnPC_2D)  { display2DPlot( iSelected ); }
    if (event.getSource() == btnPlot)   { display_plot( iSelected ); }

    if (event.getSource() == btnCSV)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _CSV;
      sName   = new String( rb[iSelected].getText() );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".csv");
    }

    if (event.getSource() == btnXML)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _XML;

      sName   = new String( rb[iSelected].getText() );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".xml");
    }

    if (event.getSource() == btnPDF)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _PDF;

      sName   = new String( rb[iSelected].getText() );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    for (int i=0; i<_TOTAL; i++)
    {
	  if (event.getSource() == rb[i])   { setData(i); }
	}

    if (event.getSource() == btnClose)  { close(); }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Close Covariance Frame"))
    {
	  if (pCov != null)
		pCov.close();
	  pCov = null;
	}

    if (sArg.equals("Close Eigenvector Frame"))
    {
      if (pEigen != null)
        pEigen.close();
      pEigen = null;
	}

	if (sArg.equals("Close Plot Control"))
	{
      if (plotXY != null)
        plotXY.close();
      plotXY = null;
	}

    if (sArg.equals("Close Cross Plot Control"))
    {
      if (plotXYPc != null)
        plotXYPc.close();
      plotXYPc = null;
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      switch (iSave)
      {
        case _CSV:
          createCSV();
          break;
        case _XML:
          createXML();
          break;
        case _PDF:
          createPDF();
          break;
	  }

      iSave = -1;

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS pcBrineExpandFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class pcBrineExpandFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}


/*
 *  @version 1.1 11/06/2015
 *  @author  John Victorine
 */
