/*
 * @pcBrineFrame.java Version 1.1 10/29/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import gui.guiSearchDirectoryFrame; // GUI Search Directory Frame

import brine.brineListStruct;       // Brine List Structure

import pc.pcDataStruct;             // Brine List Structure
import pc.pcPlotFilterListStruct;   // Plot Filter List Structure

import pc.gui.pcBrinePanel;         // Brine Panel
import pc.gui.pcCovarianceFrame;    // Covariance Frame
import pc.gui.pcEigenvectorFrame;   // Eigenvector Frame
import pc.gui.pcPlotXYFrame;        // XY Plot Frame
import pc.gui.pcPlotXplotFrame;     // Plot Principal Components Scores

import plot.plotSymbolStruct;       // Symbol Data Structure
import plot.plotXYStruct;           // XY Data Structure

import Jama.*;                      // Import Jama classes

/** Class pcBrineFrame
 *  <p> This Class will create the Stiff Frame.
 *
 *  @version 1.1 10/29/2015
 *  @author  John R. Victorine
 */

public class pcBrineFrame extends JFrame implements ActionListener, Observer
{
  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  // Input Variables

  private pcDataStruct st  = null;

  // Global Variables

  private Observable              notifier  = null; // Observables Object
  private pcBrinePanel            pnl       = null; // Brine Panel
  private pcCovarianceFrame       pCov      = null; // Covariance Frame
  private pcEigenvectorFrame      pEigen    = null; // eigenvector Frame

  // -- Directory Search for PDF

  // Save Path & Filename

  private static final int _PDF = 0;
  private static final int _XML = 1;
  private static final int _CSV = 2;

  private int    iSave    = -1;

  private guiSearchDirectoryFrame pSearch   = null; // Search for Directory

  private String sPath = "";
  private String sName = "";

  // -- Filter Data By

  private pcPlotFilterListStruct  stLegend = null; // Legend Filter List Data Structure

  // -- Plot Control Data

  private String                  sTitle1  = "Kansas Geological Survey";
  private String                  sTitle2  = "Brine Data";
  private String                  sTitle3  = "";

  private String                  sXAxis   = "Sum of Cations (meq/l)";
  private int                     iXLog    = 0;
  private double                  dXMin    = 2600.0;
  private double                  dXMax    = 4200.0;
  private double                  dXIncr   = 200.0;

  private String                  sYAxis   = "Sum of Anions (meq/l)";
  private int                     iYLog    = 0;
  private double                  dYMin    = 2600.0;
  private double                  dYMax    = 4200.0;
  private double                  dYIncr   = 200.0;

  // -- Plot Classes

  private plotXYStruct            stPlotXY = null; // XY Plot Data Structure
  private pcPlotXYFrame           plotXY   = null; // XY Plot Frame

  private plotXYStruct            stPlotPc = null;
  private pcPlotXplotFrame        plotXYPc = null; //

  private brineListStruct         stList   = null;

  // Global Widgets

  private JButton btnClose = null;
  private JButton btnCoval = null;
  private JButton btnEigen = null;
  private JButton btnPC_2D = null;
  private JButton btnPlot  = null;

  private JButton btnPDF   = null;
  private JButton btnCSV   = null;
  private JButton btnXML   = null;

  /** Constructor pcBrineFrame()
   *  <p> This is the Constructor for this class.
   *  @param st  = Brine Plot Data Structure
   */

  public pcBrineFrame( pcDataStruct st )
  {
    try
    {
      this.st = st;

      setLimits();

      jbInit();

      addWindowListener( new pcBrineFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int     iWidth   = 650;
	JPanel  pnlTop   = new JPanel();
    JPanel  pnlBTN   = new JPanel();

    JButton btnI     = new JButton();
    JButton btnII    = new JButton();
    JButton btnIII   = new JButton();
    JButton btnIV    = new JButton();

    // Create an Observable

    notifier = new pcBrineFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Brine Data showing data greater than 1.5 standard deviations");
    this.getContentPane().setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Dialog");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    btnCoval = new JButton(createImageIcon(cmn.cmnStruct.COVAL));
    btnCoval.setPreferredSize(new Dimension(29, 29));
    btnCoval.setToolTipText("Display Covalence Matrix");
    btnCoval.addActionListener(this);

    btnEigen = new JButton(createImageIcon(cmn.cmnStruct.EIGEN));
    btnEigen.setPreferredSize(new Dimension(29, 29));
    btnEigen.setToolTipText("Display Eigenvalues & Eigenvector");
    btnEigen.addActionListener(this);

    btnPC_2D = new JButton(createImageIcon(cmn.cmnStruct.PC2D));
    btnPC_2D.setPreferredSize(new Dimension(29, 29));
    btnPC_2D.setToolTipText("Display Principal Components 2D Crossplot");
    btnPC_2D.addActionListener(this);

    btnII.setPreferredSize(new Dimension(4, 35));
    btnII.setText("");

    btnPlot = new JButton(createImageIcon(cmn.cmnStruct.PLOT));
    btnPlot.setPreferredSize(new Dimension(29, 29));
    btnPlot.setToolTipText("Display Anions vs. Cations Plot");
    btnPlot.addActionListener(this);

    btnIII.setPreferredSize(new Dimension(4, 35));
    btnIII.setText("");

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Save all Panels Displayed as PNG Images");
    btnPDF.addActionListener(this);

    btnCSV = new JButton(createImageIcon(cmn.cmnStruct.CSV));
    btnCSV.setPreferredSize(new Dimension(29, 29));
    btnCSV.setToolTipText("Save Selected Data as CSV");
    btnCSV.addActionListener(this);

    btnXML = new JButton(createImageIcon(cmn.cmnStruct.XML));
    btnXML.setPreferredSize(new Dimension(29, 29));
    btnXML.setToolTipText("Save Selected Data as XML");
    btnXML.addActionListener(this);

    // Center Panel

    pnl = new pcBrinePanel( notifier, st );

    // Attach Widgets

    this.getContentPane().add(pnlTop,  BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                 BorderLayout.WEST);
    pnlBTN.add(btnClose,               null);
    pnlBTN.add(btnI,                   null);
    pnlBTN.add(btnCoval,               null);
    pnlBTN.add(btnEigen,               null);
    pnlBTN.add(btnPC_2D,               null);
    pnlBTN.add(btnII,                  null);
    pnlBTN.add(btnPlot,                null);
    pnlBTN.add(btnIII,                 null);
    pnlBTN.add(btnCSV,                 null);
    pnlBTN.add(btnXML,                 null);
    pnlBTN.add(btnPDF,                 null);

    this.getContentPane().add(pnl,     BorderLayout.CENTER);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    iWidth = pnl.getWidth();
    if (iWidth == 0)
      iWidth = 650;

    this.setSize(new Dimension(iWidth, d.height-60));

//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);
    this.setLocation( 10, 10 );

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** METHOD pcBrineFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class pcBrineFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
      close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    st       = null;
    notifier = null;

    sPath    = null;
    sName    = null;

    if (pnl != null)
      pnl.close();
    pnl = null;

    if (pCov != null)
      pCov.close();
    pCov = null;

    if (pEigen != null)
      pEigen.close();
    pEigen = null;

    if (pSearch != null)
      pSearch.close();
    pSearch = null;

    if (stPlotXY != null)
      stPlotXY.delete();
    stPlotXY   = null;

    if (plotXY != null)
      plotXY.close();
    plotXY = null;

    if (stPlotPc != null)
      stPlotPc.delete();
    stPlotPc = null;

    if (plotXYPc != null)
      plotXYPc.close();
    plotXYPc = null;

    if (stLegend != null)
      stLegend.delete();
    stLegend = null;

    stList   = null;

    btnClose     = null;
    btnCoval     = null;
    btnEigen     = null;
    btnPC_2D     = null;
    btnPlot      = null;

    btnPDF       = null;
    btnCSV       = null;
    btnXML       = null;

    dispose();
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will set the Brine List Data Structure so the normalized
   *      data can be applied to the original data
   *  @param stList = Brine List Structure
   */

  public void setData(brineListStruct stList) {	this.stList = stList; }

  /** Method setLimits()
   * <p> This method will set the Limits for plotting
   */

  private void setLimits()
  {
    double dLimits[] = null;

	if (st != null)
	{
	  dLimits = pc.pcUtility.getLimits( st );
      dXMin   = dLimits[0];
      dXMax   = dLimits[1];
      dXIncr  = dLimits[2];
      dYMin   = dLimits[3];
      dYMax   = dLimits[4];
      dYIncr  = dLimits[5];
	}
  }

  /* ======================================================================= *
   * --------------------------- DISPLAY FRAMES ---------------------------- *
   * ======================================================================= */

  /** Method buildCovalenceFrame()
   *  <p> This method will build the Covalence Table Frame
   */

  private void buildCovalenceFrame()
  {
    Matrix m        = null;
    double data[][] = null;

    if (pCov != null)
      pCov.close();
    pCov = null;

	if (st != null)
	{
	  m    = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  data = m.getArray();
	  pCov = new pcCovarianceFrame( notifier, st, data );
	}
  }

  /** Method buildEigenvectorFrame()
   *  <p> This method will build the Eigenvector Table Frame
   */

  private void buildEigenvectorFrame()
  {
    Matrix m        = null;
    Matrix evectors = null;
    EigenvalueDecomposition eigen = null;
    double data[][] = null;
    double dEigen[] = null;

    if (pEigen != null)
      pEigen.close();
    pEigen = null;

	if (st != null)
	{
	  m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
      dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	  evectors = eigen.getV();				  // Get Eigenvectors

	  data = evectors.getArray();
	  pEigen = new pcEigenvectorFrame( notifier, st, data, dEigen );
	}
  }


  /** Method void display2DPlot()
   *  <p> This method will build the 2D Principal Component Crossplot
   */

  private void display2DPlot()
  {
    Matrix m              = null;
    Matrix evectors       = null;
    EigenvalueDecomposition eigen = null;
    double data[][]       = null;
    double dVector[][]    = null;
    String sVector[]      = null;

    if (plotXYPc != null)
      plotXYPc.close();
    plotXYPc = null;

    m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
    eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
    evectors = eigen.getV();			  // Get Eigenvectors

    data     = evectors.getArray();

	dVector = new double[st.iColumns][2];
	sVector = new String[st.iColumns];

	for (int i=0; i<st.iColumns; i++)
	{
	  dVector[i][1] = data[i][st.iColumns-2];
	  dVector[i][0] = data[i][st.iColumns-1];
	  sVector[i]    = new String(CURVES[st.iBrine[i][1]][1]);
	}

	data = computePC_2D();

	stPlotPc = pc.pcPlotUtility.setPlotStruct( st.iRows, 2, data );
    plotXYPc = new pcPlotXplotFrame( notifier, 0, stPlotPc );
    plotXYPc.setVectors(st.iColumns, dVector, sVector);
  }

  /** Method computePC_2D()
   *  <p> This method will build the 2D Principal Component Crossplot
   *  @return dTemp = The first 2 principal components
   */

  private double[][] computePC_2D()
  {
    Matrix m         = null;
    Matrix evectors  = null;
    Matrix evectorsT = null;
    EigenvalueDecomposition eigen = null;
    double data[][]  = null;
    double dTemp[][] = null;
    Matrix pcV       = null;
    Matrix mAdjust   = null;
    Matrix mAdjustT  = null;
    Matrix pcAdjust  = null;
    Matrix pcAdjustT = null;
    double dAdjust[][] = null;
    double dEigen[]  = null;
    int    icnt      = 0;

	if (st != null)
	{
	  m        = pc.pcUtility.covariance( st.iRows, st.iColumns, st.dAdjust );
	  eigen    = m.eig();					  // Compute Eigenvalues & Eigenvectors
      dEigen   = eigen.getRealEigenvalues();  // Get Eigenvalues
	  evectors = eigen.getV();				  // Get Eigenvectors

	  data     = evectors.getArray();

	  dTemp = new double[st.iColumns][2];
	  for (int i=0; i<st.iColumns; i++)
	  {
		icnt = 0;
		for (int j=st.iColumns-2; j<st.iColumns; j++)
		{
          dTemp[i][icnt] = data[i][j];
          icnt++;
		}
	  }

      dAdjust = new double[st.iRows][st.iColumns];
      for (int i=0; i<st.iRows; i++)
      {
		for (int j=0; j<st.iColumns; j++)
		{
		  dAdjust[i][j] = st.dAdjust[i][j] / st.dSD[j];
		}
	  }

      pcV       = pc.pcUtility.setMatrix( st.iColumns, 2, dTemp );
      mAdjust   = pc.pcUtility.setMatrix( st.iRows, st.iColumns, dAdjust );
      mAdjustT  = mAdjust.transpose();

      pcAdjust  = evectors.times(mAdjustT);
      pcAdjustT = pcAdjust.transpose();

      dTemp     = pcAdjustT.getArray();

//	  for (int i=0; i<st.iRows; i++)
//	  {
//  	    System.out.println(i+" "+dTemp[i][0]+" "+dTemp[i][1]);
//	  }
	}

	return (dTemp);
  }

  /* ======================================================================= *
   * ----------------------------- Save Plots ------------------------------ *
   * ======================================================================= */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   *

  private void searchFrame()
  {
	if (sPath.length() == 0)
      sPath = pc.io.pcIO.getHomeDir();

    sName   = new String( "m" );
    pSearch = new guiSearchDirectoryFrame(notifier, sPath, "m", ".png");
  }
*/
  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void createPDF()
  {
	if (pnl != null)
	{
      pc.io.pcIO_Files.createImage( sPath, sName + "_brine", pnl.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName  + "_brine" );
	}

    if (pCov != null)
    {
      pc.io.pcIO_Files.createImage( sPath, sName + "_cov", pCov.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_cov" );
	}

	if (pEigen != null)
	{
      pc.io.pcIO_Files.createImage( sPath, sName + "_eign", pEigen.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_eign" );
	}

	if (plotXY != null)
	{
	  pc.io.pcIO_Files.createImage( sPath, sName + "_plot", plotXY.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_plot" );
	}

    if (plotXYPc != null)
    {
	  pc.io.pcIO_Files.createImage( sPath, sName + "_Xplot", plotXYPc.getImage() );
      pc.io.pcIO_Files.createHTML( sPath, sName + "_Xplot" );
	}
  }

  /* ===================================================================== *
   * -------------------------- CREATE FILES ----------------------------- *
   * ===================================================================== */

  /** Method createXML()
   *  <p> This method will create a ASCII XML File of Production Data
   */

  private void createXML()
  {
	String sXML = "";
	brineListStruct stNew  = null;

    stNew = brine.brineUtility.copyList( stList );

	sXML  = brine.io.WriteBrineXMLFile.buildXML( stNew );
	brine.io.WriteBrineXMLFile.write( sPath, sName+".xml", sXML );

	if (stNew != null)
	  stNew.delete();
	stNew = null;
  }

  /** Method createCSV()
   *  <p> This method will create a ASCII CSV File of Production Data
   */

  private void createCSV()
  {
	String          sCSV = "";
	brineListStruct stNew  = null;

    stNew = brine.brineUtility.copyList( stList );

	sCSV  = brine.io.WriteBrineCSVFile.buildCSV( stNew );
	brine.io.WriteBrineCSVFile.write( sPath, sName+".csv", sCSV );

	if (stNew != null)
	  stNew.delete();
	stNew = null;
  }

  /* ===================================================================== *
   * ------------------------- PLOT METHODS ------------------------------ *
   * ===================================================================== */

  /** Method display_plot()
   *  <p> This method will create the 3D Plot
   */

  private void display_plot()
  {
	if (plotXY != null)
	  plotXY.close();
	plotXY = null;

//    setPlotStruct();
    stPlotXY = pc.pcPlotUtility.setPlotStruct( st,
	                                           sTitle1, sTitle2, sTitle3,
	                                           sXAxis, dXMin, dXMax, dXIncr,
	                                           sYAxis, dYMin, dYMax, dYIncr );

    plotXY = new pcPlotXYFrame( notifier, 0, stPlotXY );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnCoval)  { buildCovalenceFrame(); }
    if (event.getSource() == btnEigen)  { buildEigenvectorFrame(); }
    if (event.getSource() == btnPC_2D)  { display2DPlot(); }
    if (event.getSource() == btnPlot)   { display_plot(); }

    if (event.getSource() == btnCSV)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _CSV;
      sName   = new String( "brine" );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".csv");
    }

    if (event.getSource() == btnXML)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _XML;

      sName   = new String( "brine" );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".xml");
    }

    if (event.getSource() == btnPDF)
    {
  	  if (sPath.length() == 0)
        sPath = pc.io.pcIO.getHomeDir();

      iSave   = _PDF;

      sName   = new String( "brine" );
      pSearch = new guiSearchDirectoryFrame(notifier, sPath, sName, ".png");
    }

    if (event.getSource() == btnClose)  { close(); }
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Close Covariance Frame"))
    {
	  if (pCov != null)
		pCov.close();
	  pCov = null;
	}

    if (sArg.equals("Close Eigenvector Frame"))
    {
      if (pEigen != null)
        pEigen.close();
      pEigen = null;
	}

	if (sArg.equals("Close Plot Control"))
	{
      if (plotXY != null)
        plotXY.close();
      plotXY = null;
	}

    if (sArg.equals("Close Cross Plot Control"))
    {
      if (plotXYPc != null)
        plotXYPc.close();
      plotXYPc = null;
	}

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      switch (iSave)
      {
        case _CSV:
          createCSV();
          break;
        case _XML:
          createXML();
          break;
        case _PDF:
          createPDF();
          break;
	  }

      iSave = -1;

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS pcBrineFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class pcBrineFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}


/*
 *  @version 1.1 10/29/2015
 *  @author  John Victorine
 */
