/*
 * @pcEigenvectorFrame.java Version 1.1 10/29/2015
 *
 * Copyright (c) 2015 Kansas Geological Survey
 * 1930 Constant Avenue, Lawrence, Kansas, 66047, U.S.A.
 * All Rights Reserved.
 */

package pc.gui;

import java.util.Observer;
import java.util.Observable;
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

import gui.guiSearchDirectoryFrame; // GUI Search Directory Frame

import pc.pcDataStruct;             // Brine List Structure
import pc.gui.pcCompPanel;          // Brine Panel
import pc.gui.pcEigenPanel;         // Eigen Values Table

/** Class pcEigenvectorFrame
 *  <p> This Class will create the Stiff Frame.
 *
 *  @version 1.1 10/29/2015
 *  @author  John R. Victorine
 */

public class pcEigenvectorFrame extends JFrame implements ActionListener, Observer
{
  public static final String CURVES[][] = brine.brineStandardTools.CURVES;

  // Input Variables

  private Observable   pNotifier = null;  // Observables Object
  private pcDataStruct st        = null;
  private double       data[][]  = null;
  private double       dEigen[]  = null;

  // Global Variables

  private Observable   notifier  = null;  // Observables Object
  private pcCompPanel  pnl       = null;  // Brine Panel
  private pcEigenPanel pEigen    = null;

  private guiSearchDirectoryFrame pSearch = null; // Search for Directory

  private String       sPath     = "";
  private String       sName     = "";

  // -- Table Variables

  private String       sLabels[] = null;
  private int          iColumns  = 0;
  private String       sColumn[] = null;
  private int          iLength[] = null;

  // Global Widgets

  private JPanel  panel    = new JPanel();

  private JButton btnClose = null;
  private JButton btnPDF   = null;

  /** Constructor pcEigenvectorFrame()
   *  <p> This is the Constructor for this class.
   *  @param st  = Brine Plot Data Structure
   */

  public pcEigenvectorFrame( pcDataStruct st, double data[][], double dEigen[] )
  {
	int len =0;
    try
    {
      this.st     = st;
      this.data   = data;
      this.dEigen = dEigen;

      if (dEigen != null) { len = dEigen.length; }
      getColumns( st, len );

      jbInit();

      addWindowListener( new pcEigenvectorFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** Constructor pcEigenvectorFrame()
   *  <p> This is the Constructor for this class.
   *  @param pNotifier = Observable Object
   *  @param st        = Brine Plot Data Structure
   *  @param data      = Covariance Matrix
   */

  public pcEigenvectorFrame( Observable pNotifier,
                             pcDataStruct st,
                             double data[][],
                             double dEigen[] )
  {
	int len =0;
    try
    {
	  this.pNotifier = pNotifier;
      this.st        = st;
      this.data      = data;
      this.dEigen    = dEigen;

      if (dEigen != null) { len = dEigen.length; }
      getColumns( st, len );

      jbInit();

      addWindowListener( new pcEigenvectorFrame_WindowListener() );
    }
    catch(Exception e)
    {
      e.printStackTrace();
    }
  }

  /** METHOD jbInit()
   *  <p> This Method will create the Frame for this class.
   *  @throws Exception
   */

  private void jbInit() throws Exception
  {
	int    iWidth    = 650;
	int    iHeight   = 0;

	JPanel pnlTop    = new JPanel();
    JPanel pnlBTN    = new JPanel();

    JButton btnI     = new JButton();

	JPanel pnlBase   = new JPanel();

    // Create an Observable

    notifier = new pcEigenvectorFrameObservable();

    // Add this Dialog to the notifier as observer

    notifier.addObserver(this);

    // Build Frame

    this.setTitle("Brine Data Eigenvector Matrix");
    this.getContentPane().setLayout(new BorderLayout());

    panel.setLayout(new BorderLayout());

    // Top Panel

    pnlTop.setLayout(new BorderLayout());

    btnClose = new JButton(createImageIcon(cmn.cmnStruct.CLOSE));
    btnClose.setPreferredSize(new Dimension(29, 29));
    btnClose.setToolTipText("Close Dialog");
    btnClose.addActionListener(this);

    // -- 1st Separator Button

    btnI.setPreferredSize(new Dimension(4, 35));
    btnI.setText("");

    btnPDF = new JButton(createImageIcon(cmn.cmnStruct.PDF));
    btnPDF.setPreferredSize(new Dimension(29, 29));
    btnPDF.setToolTipText("Save this Panel as a PNG Image");
    btnPDF.addActionListener(this);

    // Center Panel

    pnl = new pcCompPanel( sLabels, data, iColumns, sColumn, iLength );

    // Base Panel

    pnlBase.setLayout(new BorderLayout());

    pEigen = new pcEigenPanel( dEigen, iColumns, sColumn, iLength );

    // Attach Widgets

    this.getContentPane().add(pnlTop,   BorderLayout.NORTH);
    pnlTop.add(pnlBTN,                  BorderLayout.WEST);
    pnlBTN.add(btnClose,                null);
    pnlBTN.add(btnI,                    null);
    pnlBTN.add(btnPDF,                  null);

    this.getContentPane().add(panel,    BorderLayout.CENTER);
    panel.add(pnl,                      BorderLayout.CENTER);

    panel.add(pnlBase,                  BorderLayout.SOUTH);
	pnlBase.add(pEigen,                 BorderLayout.SOUTH);

    // Display the Frame

    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();

    iWidth  = pnl.getWidth();
    iHeight = pnl.getHeight()+100+60;

    if (iWidth == 0)
      iWidth = 650;

    if (iHeight == 0)
      iHeight = 700;

    this.setSize(new Dimension(iWidth, iHeight));

//    this.setLocation((d.width  - this.getSize().width) / 2,
//                     (d.height - this.getSize().height) / 2);

    this.setLocation((d.width  - this.getSize().width) - 10,
                     (d.height - this.getSize().height) - 60);

    this.setResizable(true);
    this.setVisible(true);
  }

  /** METHOD createImageIcon()
   *  <p> This Method will create the Frame for this class.
   *  @param path = the path to the image
   *  @return image icon
   */

  protected ImageIcon createImageIcon(String path)
  {
    java.net.URL imgURL = getClass().getResource(path);
    ImageIcon    image  = null;

    if (imgURL != null)
    {
      image = new ImageIcon(imgURL);
    }
    else
    {
       System.err.println("Couldn't find file: " + path);
    }

    return (image);
  }

  /* ===================================================================== *
   * -------------------------- CLOSE DIALOG ----------------------------- *
   * ===================================================================== */

  /** METHOD pcEigenvectorFrame_WindowListener()
   *  <p> This Class will handle Frame Close Action using the X Button on Frame
   */

  public class pcEigenvectorFrame_WindowListener extends WindowAdapter
  {
    public void windowClosing(WindowEvent e)
    {
	  if (pNotifier != null)
        pNotifier.notifyObservers( new String("Close Eigenvector Frame") );
	  else
	    close();
    }
  }

  /** Method close()
   * <p> This method will close all dialogs that are opened by this class.
   */

  public void close()
  {
    st        = null;
    pNotifier = null;
    notifier  = null;

    sPath     = null;
    sName     = null;

	sLabels   = null;
	data      = null;
	dEigen    = null;

    sColumn   = null;
    iLength   = null;

    if (pnl != null)
      pnl.close();
    pnl = null;

    panel     = null;

    btnClose     = null;
    btnPDF       = null;

    dispose();
  }

  /* ======================================================================= *
   * ----------------------------- GET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method getColumns()
   *  <p> This method will order the number anions and cations that can be used
   *  @param  st     = The brine List Structure
   */

  private void getColumns( pcDataStruct st, int len )
  {
	int iCount   = 0;

	if (st != null)
	{
	  iColumns = len+1;
      sColumn  = new String[iColumns];
      sLabels  = new String[st.iColumns];
      iLength  = new int[iColumns];

      sColumn[iCount] = "";
      iLength[iCount] = 8;
      iCount++;

      for (int i=1; i<iColumns; i++)
      {
		sColumn[iCount] = new String( ""+i );
        iLength[iCount] = 8;
        iCount++;
	  }

      for (int i=0; i<st.iColumns; i++)
      {
		sLabels[i] = new String( CURVES[st.iBrine[i][1]][1] );
	  }
	}
  }

  /** Method getImage()
   * <p> This method will convert this plot to a buffered image
   * @return image = profile plot buffered image
   */

  public BufferedImage getImage()
  {
    BufferedImage image = null;

	image = plot.plotBufferedWidget.componentToImage( panel, true );

	return (image);
  }

  /* ======================================================================= *
   * ----------------------------- SET METHODS ----------------------------- *
   * ======================================================================= */

  /** Method setData()
   *  <p> This method will reload the data.
   * @param st = The Zonation Data Structure
   */

  public void setData( double data[][], double dEigen[] )
  {
	if (pnl != null)
	{
	  pnl.setData( sLabels, data );
	}

	if (pEigen != null)
	{
	  pEigen.setData( dEigen );
	}
  }

  /* ======================================================================= *
   * --------------------------- DISPLAY FRAMES ---------------------------- *
   * ======================================================================= */

  /* ======================================================================= *
   * ----------------------------- Save Plots ------------------------------ *
   * ======================================================================= */

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Profile Plot
   */

  private void searchFrame()
  {
	if (sPath.length() == 0)
      sPath = pc.io.pcIO.getHomeDir();

    sName   = new String( "eigen" );
    pSearch = new guiSearchDirectoryFrame(notifier, sPath, "eigen", ".png");
  }

  /** Method createPDF()
   *  <p> This method will create a PDF Image of Panel
   */

  private void createPDF()
  {
    pc.io.pcIO_Files.createImage( sPath, sName, getImage() );
    pc.io.pcIO_Files.createHTML( sPath, sName );
  }

  /* ===================================================================== *
   * ---------------------------- ACTIONS -------------------------------- *
   * ===================================================================== */

  /** METHOD actionPerformed()
   *  <p> This Method will handle all the actions within the Frame.
   *  @param event - Action Event
   */

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == btnPDF)    { searchFrame(); }

    if (event.getSource() == btnClose)
    {
	  if (pNotifier != null)
        pNotifier.notifyObservers( new String("Close Eigenvector Frame") );
	  else
	    close();
	}
  }

  /* ===================================================================== *
   * ------------------------ OBSERVER ACTIONS --------------------------- *
   * ===================================================================== */

  /** Method update()
   * <p> This method will handle the Observable Actions
   * @param obj = The Observable making the call
   * @param arg = The returned object
   */

  public void update(Observable obj, Object arg)
  {
    String sArg = new String( (String) arg);

    if (sArg.equals("Directory Path Selected"))
    {
      if (pSearch != null)
      {
        sPath = new String( pSearch.getPath() );
        sName = new String( pSearch.getFile() );
      }

      createPDF();

      if (pSearch != null)
        pSearch.close();
      pSearch = null;
	}
  }
}

/** CLASS pcEigenvectorFrameObservable()
  *  <p> This Class will handle Observable Actions when action occurs on a
  *      frame or panel
  */

class pcEigenvectorFrameObservable extends Observable
{
  public void notifyObservers(Object b)
  {
    setChanged();
    super.notifyObservers(b);
  }
}


/*
 *  @version 1.1 10/29/2015
 *  @author  John Victorine
 */
